#////////1/////////2/////////3/////////4/////////5/////////6/////////7/////////8
# Name        :
# Author      : Avi
# Revision    : $Revision: #10 $
#
# Copyright 2009- ECMWF.
# This software is licensed under the terms of the Apache Licence version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
# In applying this licence, ECMWF does not waive the privileges and immunities
# granted to it by virtue of its status as an intergovernmental organisation
# nor does it submit to any jurisdiction.
#////////1/////////2/////////3/////////4/////////5/////////6/////////7/////////8
#  code for testing creation of defs file in python
import os
import ecflow

print(ecflow.__doc__)
print(ecflow.Defs.__doc__)
    
from ecflow import Suite, Family, Task, Defs, Clock, DState, PartExpression, Variable, Limit, InLimit, \
                   Date, Day, Event, Meter, Label, Autocancel, Days, TimeSlot, TimeSeries, Style, State, \
                   RepeatString, RepeatDate, RepeatDateTime, RepeatInteger, RepeatDay, RepeatEnumerated, \
                   Verify, PrintStyle, Time, Today, Late, Cron, Client, debug_build,Ecf
import ecflow_test_util as Test

if __name__ == "__main__":
    
    Test.print_test_start(os.path.basename(__file__))

    suite = Suite("s1")
    assert (isinstance(suite, ecflow.Suite)), "Expected suite"
    assert (not isinstance(suite, ecflow.Family)), "Expected suite"
    assert (not isinstance(suite, ecflow.Task)), "Expected suite"
 
    family = Family("f1")
    assert (isinstance(family, ecflow.Family)), "Expected Family"
    assert (not isinstance(family, ecflow.Suite)), "Expected Family"
    assert (not isinstance(family, ecflow.Task)), "Expected Family"
    suite.add_family(family)

    task = Task("f1")
    assert (isinstance(task, ecflow.Task)), "Expected Task"
    assert (not isinstance(task, ecflow.Suite)), "Expected Task"
    assert (not isinstance(task, ecflow.Family)), "Expected Task"
    family.add_task(task)

    
    defs = Defs()
    defs2 = Defs()
    assert defs == defs2, "Empty defs should be equal"
 
    assert defs.get_state() == ecflow.State.unknown, "Expected default state to be unknown"
    defs.add_suite(suite);  # add existing suite to defs
 
    suite = defs.add_suite("s2");
    suite.add_variable(Variable("ECF_HOME","/tmp/"))
    suite.add_variable("ECF_URL_CMD", "${BROWSER:=firefox} -new-tab %ECF_URL_BASE%/%ECF_URL%")
    suite.add_variable("ECF_URL_BASE","http://www.ecmwf.int")
    suite.add_variable("ECF_URL","publications/manuals/sms")
    suite.add_limit( Limit("limitName", 10) )
    suite.add_limit( "limitName_2", 10 )
    assert suite.begun() == False, "Suite should not have begun"

    clock = Clock(1,1,2010,False);
    clock.set_gain(1,10,True);
    suite.add_clock(clock);

    assert defs != defs2, "Defs should no longer compare as they are different"
    
    defs.add_extern("/path/to/task");
    defs.add_extern("/fred/bill:event_name");
    
    family = suite.add_family("f1")
    family.add_defstatus( DState.active );

    task = family.add_task("t1")
    task.add_trigger( "t2 == active" )
    task.add_complete( "t2 == complete" )
    task.add_autocancel( Autocancel(3) );

    task2 = family.add_task("t2")
    task2.add_defstatus(  DState.complete );
    task2.add_part_trigger(  PartExpression("t1 == complete") )
    task2.add_part_trigger(  PartExpression("t1 == active",True) )  #  for long and/or expressions, subsequent expr must be and/or
    task2.add_part_complete( PartExpression("t1 == complete") )
    task2.add_part_complete( PartExpression("t1 == active",False) )  #  for long and/or expressions, subsequent expr must be and/or
    task2.add_label(      Label("labelname", "label1") )
    task2.add_label(      "labelname2", "label1")
    task2.add_inlimit(    InLimit("limitName","/s1/f1",2) )
    task2.add_inlimit(    InLimit("limitName1","/s1/f1",2,True) )
    task2.add_inlimit(    "limitName2" )
    task2.add_inlimit(    "limitName3","path" )
    task2.add_inlimit(    "limitName4","path",1 )
    task2.add_inlimit(    "limitName5","path",1,True )
    task2.add_event(      10 )
    task2.add_event(      Event(10,"Eventname") )
    task2.add_meter(      Meter("metername",0,100,50) )
    task2.add_meter(      "metername2",0,100,50 )
    task2.add_date(       Date(1,1,2010) )
    task2.add_date(       1,2,2010 )
    task2.add_day(        Day(Days.sunday) )
    task2.add_day(        Days.monday )
    task2.add_autocancel( Autocancel(TimeSlot(20,10),True) )

    task2.add_today(      0,10 )
    task2.add_today(      Today( 0,59, True ))
    task2.add_today(      Today(TimeSlot(20,10)) )
    task2.add_today(      Today(TimeSlot(20,20),False)) 
    start = TimeSlot(0,0)
    finish = TimeSlot(23,0)
    incr = TimeSlot(0,30)
    ts = TimeSeries( start, finish, incr, True);
    task2.add_today(      Today(ts) )

    task2.add_time(       20,10 )
    task2.add_time(       Time(20,12,True) )
    task2.add_time(       Time(TimeSlot(20,12))) 
    task2.add_time(       Time(TimeSlot(20,20),True)) 
    task2.add_time(       Time(ts) )
    
    late = Late()
    late.submitted( 20,10 )
    late.active(TimeSlot(20,10))
    late.complete(TimeSlot(20,10),True)
    task2.add_late(  late )
        
    cron = Cron()
    cron.set_week_days( [0,1,2,3,4,5,6] )
    cron.set_days_of_month(  [1,2,3,4,5,6] )
    cron.set_months( [1,2,3,4,5,6] )
    cron.set_time_series( ts )
    task2.add_cron( cron );
    
    task3 = family.add_task("t3")
    task3.add_repeat( RepeatDate("testDate",20100111,20100115,2) )
    task3.add_defstatus( DState.queued );
    task3.add_verify( Verify(State.complete, 1) )
    task3.add_autocancel( Autocancel(20,10,False) )
    task3.add_label( Label("label_name","label_value") )

    task3_1 = family.add_task("t3_1")
    task3_1.add_repeat( RepeatDate("testDate",20100111,20100115) )

    task3_2 = family.add_task("t3_2")
    task3_2.add_repeat(RepeatDateTime("testDateTime", "20100111T000000", "20100115T000000"))


    task4 = family.add_task("t4")
    task4.add_repeat( RepeatInteger("testInteger",0,100,2) )
    task4.add_defstatus( DState.aborted );
    
    task4_1 = family.add_task("t4_1")
    task4_1.add_repeat( RepeatInteger("testInteger",0,100) )
 
    
    task5 = family.add_task("t5")
    task5.add_repeat( RepeatEnumerated("testInteger", ["red", "green", "blue" ] ) )
    task5.add_defstatus( DState.submitted );
        
    task6 = family.add_task("t6")
    task6.add_repeat( RepeatString("test_string",  ["a", "b", "c" ] ) )
    task6.add_defstatus( DState.suspended );
    
    task7 = family.add_task("t7")
    task7.add_repeat( RepeatDay(2) )
    task7.add_defstatus( DState.active );
    
    task8 = family.add_task("t8")
    task8.add_repeat( RepeatDay() )
    print("task8.get_try_no()    :string:",task8.get_try_no())
    print("task8.get_int_try_no():int   :",task8.get_int_try_no())
    
    the_defs = task8.get_defs()
 
          
    # Test find      
    assert defs.find_suite("s1") is not None, "Expected to find suite of name s1"
    assert defs.find_suite("sffff1") is None, "Should not be able to find suite of name sfffff1"
        
    # save the defs file as a check point file and restore it again
    checkpt_file = "py_u_TestDefs.check"
    defs_file = "py_u_TestDefs.def"
    if debug_build() :
        checkpt_file = "py_u_TestDefs_debug.check"
        defs_file = "py_u_TestDefs_debug.def"
    defs.save_as_checkpt(checkpt_file);
    
    restored_checkpt_defs = Defs()
    restored_checkpt_defs.restore_from_checkpt(checkpt_file)
    
    print("Save and restore as Defs ******************************************************")
    defs.save_as_defs(defs_file)           # default is to save as DEFS
    restored_from_defs = Defs( defs_file ) # restore the defs

    defs.save_as_defs(defs_file,Style.DEFS)
    restored_from_defs = Defs( defs_file ) # restore the defs
            
    print("Save and restore using STATE ******************************************************")
    defs.save_as_defs(defs_file,Style.STATE)
    restored_from_defs = Defs( defs_file ) # restore the defs

    print("Save and restore using MIGRATE ******************************************************")
    defs.save_as_defs(defs_file,Style.MIGRATE)
    restored_from_defs = Defs( defs_file ) # restore the defs
    assert restored_checkpt_defs == restored_from_defs ,"File should be the same"

        
    print("Print in DEFS style ******************************************************")
    PrintStyle.set_style(Style.DEFS)
    the_string = str(restored_from_defs)
    assert the_string.find("defs_state") == -1, "Print in DEFS style failed"
    print(the_string)
    
    print("Print in STATE style *****************************************************")
    PrintStyle.set_style(Style.STATE)
    the_string = str(restored_from_defs)
    assert the_string.find("defs_state STATE") != -1, "Print in STATE style failed"
    print(the_string)
    
    print("Print in MIGRATE style *****************************************************")
    PrintStyle.set_style(Style.MIGRATE)
    the_string = str(restored_from_defs)
    assert the_string.find("defs_state MIGRATE") != -1, "Print in MIGRATE style failed"
    print(the_string)
     
    # Comment this out if you want to see what the file looked like
    os.remove(checkpt_file)
    os.remove(defs_file)
    
    # Test job creation failure
    msg = defs.check_job_creation()
    assert len(msg) != 0 ,"Expected job creation to fail"
    
    expected_exeption = False;
    try:
        defs.check_job_creation(throw_on_error=True) 
    except RuntimeError as e:
        expected_exeption = True
    assert expected_exeption,"expected exception"  
 
    print("All tests pass")



import os
import ecflow

defs = ecflow.Defs()
suite = defs.add_suite("data_acquisition")
suite.add_repeat(ecflow.RepeatDay(1))
suite.add_variable("ECF_HOME", os.getenv("HOME") + "/course")
suite.add_variable("ECF_INCLUDE", os.getenv("HOME") + "/course")
suite.add_variable("ECF_FILES", os.getenv("HOME") + "/course/data")
suite.add_variable("SLEEP", "2")
for city in (
    "Exeter",
    "Toulouse",
    "Offenbach",
    "Washington",
    "Tokyo",
    "Melbourne",
    "Montreal",
):
    fcity = suite.add_family(city)
    fcity.add_task("archive")
    for obs_type in ("observations", "fields", "images"):
        type_fam = fcity.add_family(obs_type)
        if city in ("Exeter", "Toulouse", "Offenbach"):
            type_fam.add_time("00:00 23:00 01:00")
        if city in ("Washington"):
            type_fam.add_time("00:00 23:00 03:00")
        if city in ("Tokyo"):
            type_fam.add_time("12:00")
        if city in ("Melbourne"):
            type_fam.add_day("monday")
        if city in ("Montreal"):
            type_fam.add_date(1, 0, 0)

        type_fam.add_task("get")
        type_fam.add_task("process").add_trigger("get eq complete")
        type_fam.add_task("store").add_trigger("get eq complete")



#!/usr/bin/env python2.7
import os
import ecflow 
 
print "Creating suite definition"   
defs = ecflow.Defs()
suite = defs.add_suite("test")
suite.add_variable("ECF_INCLUDE", os.getenv("HOME") + "/course")
suite.add_variable("ECF_HOME",    os.getenv("HOME") + "/course")

suite.add_family("f3").add_task("t1").add_label("info","") 
print defs

print "Checking job creation: .ecf -> .job0"   
print defs.check_job_creation()

print "Saving definition to file 'test.def'"
defs.save_as_defs("test.def") 


#////////1/////////2/////////3/////////4/////////5/////////6/////////7/////////8
# Name        :
# Author      : Avi
# Revision    : $Revision: #10 $
#
# Copyright 2009- ECMWF.
# This software is licensed under the terms of the Apache Licence version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
# In applying this licence, ECMWF does not waive the privileges and immunities
# granted to it by virtue of its status as an intergovernmental organisation
# nor does it submit to any jurisdiction.
#////////1/////////2/////////3/////////4/////////5/////////6/////////7/////////8

# This file is used to test the python traversal routines
# This is done by loading from disk a defs file.
# traversing this defs file, and creating another defs file
# then comparing the two. If traversal was correct. the file should be the same

import ecflow
import ecflow_test_util as Test
import os,fnmatch,sys
            
class Indentor:
    """This class manages indentation, for use with context manager
    It is used to correctly indent the definition node tree hierarchy
    """
    _index = 0
    def __init__(self):
        Indentor._index += 1
    def __del__(self):
        Indentor._index -= 1
    @classmethod
    def indent(cls,the_file):
        for i in range(Indentor._index):
            the_file.write(' ')
            
class DefsTraverser:
    """Traverse the ecflow.Defs definition and write to file.
    
    This demonstrates that all nodes in the node tree and all attributes are accessible.
    Additionally the state data is also accessible. This class will write state data as
    comments. If the definition was returned from the server, it allows access to latest 
    snapshot of the state data held in the server. 
    """
    def __init__(self,defs,file_name):
        assert (isinstance(defs,ecflow.Defs)),"Expected ecflow.Defs as first argument"
        assert (isinstance(file_name,str)),"Expected a string argument. Representing a file name"
        self.__defs = defs
        self.__file = open(file_name, 'w')
        
    def write_to_file(self):
        for extern in self.__defs.externs:
            self.__writeln("extern " + extern)
        for suite in self.__defs:
            self.__write("suite ")
            self.__print_node(suite)
            clock = suite.get_clock()
            if clock:
                indent = Indentor()
                self.__writeln(str(clock))
                del indent
            self.__print_nc(suite)
            self.__writeln("endsuite")  
        self.__file.close()

    def __print_nc(self,node_container):
        indent = Indentor()
        for node in node_container:
            if isinstance(node, ecflow.Task):
                self.__write("task ")
                self.__print_node(node)
                self.__print_alias(node)
            else: 
                self.__write("family ")
                self.__print_node(node)
                self.__print_nc(node)
                self.__writeln("endfamily")
        del indent

    def __print_alias(self,task):
       indent = Indentor()
       for alias in task:
           self.__write("alias ")
           self.__print_node(alias)
           self.__writeln("endalias")
       del indent

    def __print_node(self,node):
        self.__file.write(node.name() + " # state:" + str(node.get_state()) + "\n")
        
        indent = Indentor()
        defStatus = node.get_defstatus()
        if defStatus != ecflow.DState.queued: 
            self.__writeln("defstatus " + str(defStatus))
            
        autocancel = node.get_autocancel()
        if autocancel: self.__writeln(str(autocancel))

        autoarchive = node.get_autoarchive()
        if autoarchive: self.__writeln(str(autoarchive))

        autorestore = node.get_autorestore()
        if autorestore: self.__writeln(str(autorestore))
        
        repeat = node.get_repeat()
        if not repeat.empty(): self.__writeln(str(repeat)  + " # value: " + str(repeat.value()))
    
        late = node.get_late()
        if late: self.__writeln(str(late) + " # is_late: " + str(late.is_late()))

        complete_expr = node.get_complete()
        if complete_expr:
            for part_expr in complete_expr.parts:
                trig = "complete "
                if part_expr.and_expr(): trig = trig + "-a "
                if part_expr.or_expr():  trig = trig + "-o "
                self.__write(trig)
                self.__file.write( part_expr.get_expression() + "\n")
        trigger_expr = node.get_trigger()
        if trigger_expr:
            for part_expr in trigger_expr.parts:
                trig = "trigger "
                if part_expr.and_expr(): trig = trig + "-a "
                if part_expr.or_expr():  trig = trig + "-o "
                self.__write(trig)
                self.__file.write( part_expr.get_expression() + "\n")
                
        for var in node.variables:    self.__writeln("edit " + var.name() + " '" + var.value() + "'")
        for meter in node.meters:     self.__writeln(str(meter) + " # value: " + str(meter.value()))
        for event in node.events:     self.__writeln(str(event) + " # value: " + str(event.value()))
        for label in node.labels:     self.__writeln(str(label) + " # value: " + label.new_value())
        for limit in node.limits:     self.__writeln(str(limit) + " # value: " + str(limit.value()))
        for inlimit in node.inlimits: self.__writeln(str(inlimit))
        for the_time in node.times:   self.__writeln(str(the_time))
        for today in node.todays:     self.__writeln(str(today))   
        for date in node.dates:       self.__writeln(str(date))  
        for day in node.days:         self.__writeln(str(day))   
        for cron in node.crons:       self.__writeln(str(cron))    
        for verify in node.verifies:  self.__writeln(str(verify))
        for zombie in node.zombies:   self.__writeln(str(zombie))
        for queue in node.queues:     self.__writeln(str(queue))
        for generic in node.generics: self.__writeln(str(generic))
        
        del indent

    def __write(self,the_string):
        Indentor.indent(self.__file)
        self.__file.write(the_string)

    def __writeln(self,the_string):
        Indentor.indent(self.__file)
        self.__file.write(the_string + "\n")
                                   
def check_traversal(path_to_def):
    # Open a def on disk *and* load into memory
    reference_def = ecflow.Defs( path_to_def )
         
    # traverse the opened def and write it out again
    file_name = "copy.def"
    traverser = DefsTraverser(reference_def,file_name)
    traverser.write_to_file()
         
    # restore the defs we create via traversal. If traversal was good it should
    # be the same as def on disk.
    try:
        traversed_def = ecflow.Defs( file_name )
    except RuntimeError as e: 
        print("Could not parse file " + file_name + "\n" + str(e))
        sys.exit(1)
         
    # compare the two defs
    ecflow.Ecf.set_debug_equality(True)
    defs_equal = (traversed_def == reference_def)
    if not defs_equal:
        print(str(path_to_def) + " FAILED ")
        print("The traversed defs=========\n" + str(traversed_def) + "\nnot the same as reference def============\n" + str(reference_def))
        print("===================== " + file_name + " ====================================")
        the_traversed_def_on_disk = open(file_name)
        for line in the_traversed_def_on_disk:
            print(line)
            
        assert defs_equal, "Failed: ---"
      
        # Notice: this path does not delete file_name, left for analysis of failure
    else:
        print(str(path_to_def) + " PASSED ")
        os.remove(file_name)        
        

def all_files(root, patterns='*', single_level=False, yield_folders=False):
    """Expand patterns from semi-colon separated string to list"""
    patterns = patterns.split(';')
    for path, subdirs, files in os.walk(root):
        if yield_folders:
            files.extend(subdirs)
        files.sort()
        for name in files:
            for pattern in patterns:
                if fnmatch.fnmatch(name,pattern):
                    yield os.path.join(path, name)
                    break
        if single_level:
            break    
        
if __name__ == "__main__":
    Test.print_test_start(os.path.basename(__file__))
 
    cwd = os.getcwd()
    #print cwd
    #print "split = " + str(os.path.split(cwd))
    #print "basename = " + os.path.basename(cwd)
    #print "dirname = " + os.path.dirname(cwd)
    #print "splitext = " + str( os.path.splitext(cwd) )
    #print "isdir = " + str(os.path.isdir(cwd))
    
    # Traverse all the good defs in the Parser directory
    newpath = ""
    if os.path.basename(cwd) == "libs/pyext":
        newpath = os.path.join( os.path.dirname(cwd),"libs/node/parser/test/data/good_defs" )
    
    #print newpath
    for path_to_defs_file in all_files(newpath, '*.def;*.txt'):
        check_traversal(path_to_defs_file)
        
    # try the mega_def. Commented out since it takes to long
    #mega_def = os.path.join( os.path.dirname(cwd), "libs/node/parser/test/data/single_defs/mega.def")
    #check_traversal(mega_def)
    print("All Tests pass")    



import os
from ecflow import Defs, Suite, Task, Edit

print("Creating suite definition")
home = os.path.join(os.getenv("HOME"), "course")
defs = Defs(Suite("test", Edit(ECF_HOME=home), Task("t1"), Task("t2")))
print(defs)

print("Checking job creation: .ecf -> .job0")
print(defs.check_job_creation())

print("Saving definition to file 'test.def'")
defs.save_as_defs("test.def")



import ecflow

print("Client -> Server: delete, then load a new definition")
try:
    ci = ecflow.Client()
    ci.delete_all()  # clear out the server
    ci.load("test.def")  # load the definition into the server
    ci.begin_suite("test")  # start the suite
except RuntimeError as e:
    print("Failed:", e)



#////////1/////////2/////////3/////////4/////////5/////////6/////////7/////////8
# Name        :
# Author      : Avi
# Revision    : $Revision: #10 $
#
# Copyright 2009- ECMWF.
# This software is licensed under the terms of the Apache Licence version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
# In applying this licence, ECMWF does not waive the privileges and immunities
# granted to it by virtue of its status as an intergovernmental organisation
# nor does it submit to any jurisdiction.
#////////1/////////2/////////3/////////4/////////5/////////6/////////7/////////8

# This code is used in the user manual 
#

from ecflow import Suite, Task, RepeatDate
      
if __name__ == "__main__":
 
    class ExperimentalSuite(object):
        def __init__(self, start, end) :
            self.start_ = start
            self.end_ = end
            self.start_cycle_ = 12
            self.end_cycle_ = 12
            
        def generate(self) :
            x_suite = Suite("x")
            make_fam = x_suite.add_family("make")
            make_fam.add_task("build")
            make_fam.add_task("more_work")
            
            main_fam = x_suite.add_family("main")
            main_fam.add_repeat( RepeatDate("YMD",self.start_,self.end_) )
            main_fam.add_trigger( "make == complete" )
           
            previous = 0
            for FAM in ( 0, 6, 12, 18 ) :
                fam_fam = x_suite.add_family(str(FAM))
                if FAM > 0 :
                    fam_fam.add_trigger( "./" + str(previous) + " == complete " )
                   
                self.add_complete(fam_fam,FAM)
                
                fam_fam.add_task("run")
                fam_fam.add_task("run_more").add_trigger( "run == complete")
                previous = FAM
            return x_suite
           
        def add_complete(self, family, fam):
            if fam < self.start_cycle_ and fam > self.end_cycle_ :
                family.add_complete("../main:YMD eq " + str(self.start_) + " or ../main:YMD ge " + str(self.end_))  
            elif fam < self.start_cycle_  :
                family.add_complete("../main:YMD eq " + str(self.start_))            
            elif fam > self.end_cycle_    :
                family.add_complete("../main:YMD ge " + str(self.end_))
            return
 
    print(str( ExperimentalSuite(20050601,20050605).generate() ))
   
# ==========================================================================
 
# Control structure and looping
    var = "aa"
    if var in ( "a", "aa", "aaa" ) :     print("it is a kind of a ")
    elif var in ( "b", "bb", "bb" ) :    print("it is a kind of b ")
    else :                               print("it is something else ")
   
              
# ====================================================================================

    # Reuseable class for adding synoptic times
    class VarAdder(object):
        def __init__(self, node):
            self.node = node
          
        def add(self, time):
            {
               6:  lambda  self : self.add6(time),
               12: lambda  self : self.add12(time),
               18: lambda  self : self.add18(time),
               24: lambda  self : self.add24(time) 
             }.get(time, self.errorHandler)(self)
      
        def add6(self, time):  
            print("add6 " + str(time))
            self.node.add_today(17, 30)
            self.node.add_variable("ANTIME", str(time))

        def add12(self, time):  
            print("add12 " + str(time))
            self.node.add_today(19, 15)
            self.node.add_variable("ANTIME", str(time))

        def add18(self, time):  
            print("add18 " + str(time))
            self.node.add_time(1, 30)
            self.node.add_variable("ANTIME", str(time))

        def add24(self, time):  
            print("add24 " + str(time))
            self.node.add_time(3, 0)
            self.node.add_variable("ANTIME", "0")
            self.node.add_variable("DELTA_DAY", "1")
            self.node.add_variable("EXPVER", "0002")
         
        def errorHandler(self, ignore): print("invalid time " + str(ignore))

#for i in (0, 6 ,12, 18, 24):
for i in (0, 6):
    task = Task("t" + str(i))
    print(task.name())
    varAdder = VarAdder(task)
    varAdder.add(114)
    for var in task.variables: 
        print(str(var) + "\n")
    for the_time in task.times : 
        print(str(the_time) + "\n")
    for today in task.todays : 
        print(str(today) + "\n")
         
# ==========================================================================

suite = Suite("x")
previous_time = 0
for i in (0,6,12,18,24) :
    the_fam = suite.add_family(str(i))
    if i != 0:
        the_fam.add_trigger("./" + previous_time + " == complete ")
    the_fam.add_task("t1")
    the_fam.add_task("t2").add_trigger("t1 == complete")
    previous_time = str(i)

# print str(suite)



#!/usr/bin/python
#////////1/////////2/////////3/////////4/////////5/////////6/////////7/////////8
# Name        :
# Author      : Avi
# Revision    : $Revision: #10 $
#
# Copyright 2009- ECMWF.
# This software is licensed under the terms of the Apache Licence version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
# In applying this licence, ECMWF does not waive the privileges and immunities
# granted to it by virtue of its status as an intergovernmental organisation
# nor does it submit to any jurisdiction.
#////////1/////////2/////////3/////////4/////////5/////////6/////////7/////////8
# file deepcode ignore PythonIdenticalBranches: <comment the reason here>

"""Thanks https://github.com/pazz/urwidtrees
"""



import os
import re
import sys
import time

import urwid
import ecflow
mark = 'dirmark'

UPDATE_INTERVAL = 180

singles = {"queued": "Q",
           "submitted": "S",
           "active": "R",
           "complete": "C",
           "aborted": "A",
           "suspended": "P",
           "unknown": "U",
           "RUNNING": "R",
           "HALTED": "H",
           "SHUTDOWN": "S",

           "dirmark": "-",
           # "unkown": "U",
           }
global STATUS_KIND
STATUS_KIND = True


def is_expanded(item):
    return get_status(item, True) in (
        "aborted", "active", "submitted", "RUNNING", "unknown",
        "A", "R", "S", "U", "abo", "RUN", "unk",
    )


def decorate(node, parent=None, keys=None):
    if type(node) not in (ecflow.Defs, ecflow.Alias,
                          ecflow.Suite, ecflow.Family, ecflow.Task):
        if 1:
            raise Exception(type(node))
        return ""
    status = get_status(node)
    ext = ' %c [%s]' % (kinds[type(node)], status)
    num = 0

    if type(node) in (ecflow.Suite, ecflow.Family, ecflow.Task, ):
        if "%s" % node.get_defstatus() == status:
            if status != "queued":
                ext += " (defstatus)"

    if type(node) == ecflow.Defs:
        return ext
    exp = node.get_repeat()
    if exp:
        ext += "\n%s" % exp

    exp = node.get_trigger()
    if exp:
        ext += "\n%s" % exp

    exp = node.get_complete()
    if exp:
        ext += "\n%s" % exp

    exp = node.get_late()
    if exp:
        ext += "\n%s" % exp

    exp = node.get_autocancel()
    if exp:
        ext += "\n%s" % exp

    for item in node.events:
        if keys:
            parent._children[num] = LeafNode(item)
            keys.append("%s" % item)
        else:
            ext += " %s" % item
        num += 1

    for item in node.meters:
        if keys:
            parent._children[num] = LeafNode(item)
            keys.append("%s" % item)
        else:
            ext += " %s" % item
        num += 1

    for item in node.labels:
        if keys:
            parent._children[num] = LeafNode(item)
            keys.append("%s" % item)
        else:
            ext += " %s" % item
        num += 1

    for item in node.limits:
        if keys:
            parent._children[num] = LeafNode(item)
            keys.append("%s" % item)
        else:
            ext += " %s" % item
        num += 1

    for item in node.inlimits:
        if keys:
            parent._children[num] = LeafNode(item)
            keys.append("%s" % item)
        else:
            ext += " %s" % item
        num += 1

    for item in node.times:
        if keys:
            parent._children[num] = LeafNode(item)
            keys.append("%s" % item)
        else:
            ext += " %s" % item
        num += 1

    for item in node.dates:
        if keys:
            parent._children[num] = LeafNode(item)
            keys.append("%s" % item)
        else:
            ext += " %s" % item
        num += 1

    for item in node.todays:
        if keys:
            parent._children[num] = LeafNode(item)
            keys.append("%s" % item)
        else:
            ext += " %s" % item
        num += 1

    for item in node.crons:
        if keys:
            parent._children[num] = LeafNode(item)
            keys.append("%s" % item)
        else:
            ext += " %s" % item
        num += 1

    for item in node.days:
        if keys:
            parent._children[num] = LeafNode(item)
            keys.append("%s" % item)
        else:
            ext += " %s" % item
        num += 1

    for item in node.zombies:
        if keys:
            parent._children[num] = LeafNode(item)
            keys.append("%s" % item)
        else:
            ext += " %s" % item
        num += 1

    for item in node.variables:
        if keys:
            parent._children[num] = LeafNode(item)
            keys.append("%s" % item)
        else:
            ext += " %s" % item
        num += 1

    gvar = ecflow.VariableList()
    node.get_generated_variables(gvar)
    for item in gvar:
        if keys:
            parent._children[num] = LeafNode(item)
            keys.append("%s" % item)

    if keys:
        return num
    return ext


def get_name(item, full=False, parent=None, keys=None):
    if type(item) in (ContainerNode, LeafNode):
        item = item.get_value()
    if type(item) in (dict, tuple, ):
        return "#list"
    if type(item) in (str, ):
        if 1:
            raise Exception(item)
        return item
    if type(item) in (ecflow.Client, ):
        return "%s@%s" % (item.get_host(), item.get_port()) + \
               " - %s " % time.time()

    # ext = ' %c [%s]' % (kinds[type(item)], get_status(item))
    ext = decorate(item)
    if type(item) in (ecflow.Defs, ):
        return "defs" + str(ext) + " - %s " % time.time()

    if type(item) in (ecflow.Client, ):
        return "%s@%s" % (item.get_host(), item.get_port()) + \
               " - %s " % time.time()
    if full:
        return "%s" % item.name() + str(ext)
    return item.name()


def get_status(item, full=False):
    if type(item) in (ecflow.Client, ):
        item = item.get_defs()

    if type(item) in (ecflow.Defs, ):
        out = "%s" % item.get_server_state()
    elif type(item) in (dict, tuple):
        out = "unknown"
    elif type(item) == str:
        if item == dir_sep():
            out = "unknown"
        else:
            raise Exception(item, type(item))
    else:
        out = "%s" % item.get_state()

    if full:
        return out
    if STATUS_KIND == 1:
        return singles[out]
    if STATUS_KIND == 3:
        return out[:3]
    return out


class FlagNodeW(urwid.TreeWidget):
    unexpanded_icon = urwid.AttrMap(urwid.TreeWidget.unexpanded_icon, mark)
    expanded_icon = urwid.AttrMap(urwid.TreeWidget.expanded_icon, mark)

    def __init__(self, item):
        super(FlagNodeW, self).__init__(item)
        self._w = urwid.AttrWrap(self._w, None)
        self.flagged = False
        self.update_w()

    def selectable(self):
        return True

    def keypress(self, size, key):
        key = super(FlagNodeW, self).keypress(size, key)
        if key:
            key = self.unhandled_key(size, key)
        return key

    def unhandled_key(self, size, key):
        if key == " ":
            self.flagged = not self.flagged
            self.update_w()
        else:
            return key

    def update_w(self):
        if self.flagged:
            self._w.attr = 'flagged'
            self._w.focus_attr = 'focus'
        else:
            self._w.attr = 'body'
            self._w.focus_attr = 'focus'
        self._w.attr = get_status(self.get_node().get_value(), True)


class EmptyW(urwid.TreeWidget):

    def get_display_text(self):
        return ('flag', "(empty)")


class ErrorW(urwid.TreeWidget):

    def get_display_text(self):
        return ('flag', "(error)")


class NodeW(FlagNodeW):

    def __init__(self, item):
        super(NodeW, self).__init__(item)
        node = item.get_value()
        add_widget(node, self)

    def get_display_text(self):
        node = self.get_node().get_value()
        return get_name(node, True)


class ContainerNodeW(FlagNodeW):

    def __init__(self, item):
        super(ContainerNodeW, self).__init__(item)
        node = item.get_value()
        add_widget(node, self)
        self.expanded = is_expanded(node)
        self.update_expanded_icon()

    def get_display_text(self):
        node = self.get_node().get_value()
        return get_name(node, True)


class EmptyNode(urwid.TreeNode):

    def load_widget(self):
        return EmptyW(self)


class ErrorNode(urwid.TreeNode):

    def load_widget(self):
        return ErrorW(self)


class LeafNode(urwid.TreeNode):

    def __init__(self, item, parent=None):
        if type(item) == ecflow.Task:
            path = item.get_abs_node_path()
            depth = path.count(dir_sep())
            key = os.path.basename(path)
        else:
            path = "%s" % item
            depth = 0
            key = "%s" % item
        urwid.TreeNode.__init__(self, item, key=key,
                                parent=parent, depth=depth)

    def load_parent(self):
        node = self.get_value()
        if type(node) in (ecflow.Client, ecflow.Defs, ):
            path = dir_sep()
        else:
            path = node.get_abs_node_path()
        parentname, name = os.path.split(path)
        if 1:
            raise Exception
        parent = ContainerNode(self.ci.find_abs_node_path(parentname))
        parent.set_child_node(self.get_key().get_value(), self)
        return parent

    def load_widget(self):
        return NodeW(self)

    # def load_child_keys(self): return [None]


class ContainerNode(urwid.ParentNode):

    def __init__(self, item, parent=None):
        if type(item) in (ecflow.Client, ecflow.Defs, ):
            path = dir_sep()
        else:
            path = item.get_abs_node_path()
        depth = path.count(dir_sep())
        if path == dir_sep():
            depth = 0
            key = None
        else:
            key = os.path.basename(path)
        urwid.ParentNode.__init__(
            self, item, key=key, parent=parent, depth=depth)

    def load_parent(self):
        node = self.get_value()
        if type(node) in (ecflow.Client, ecflow.Defs, ):
            path = dir_sep()
        else:
            path = node.get_abs_node_path()
        parentname, name = os.path.split(path)
        if 1:
            raise Exception
        parent = ContainerNode(self.ci.find_abs_node_path(parentname))
        parent.set_child_node(self.get_key().get_value(), self)
        return parent

    def load_child_keys(self):
        keys = []
        num = 0
        node = self.get_value()
        if type(node) in (ecflow.Client, ):
            node = node.get_defs()

        if type(node) in (ecflow.Defs, ):
            for item in node.suites:
                self._children[num] = ContainerNode(item)
                keys.append(get_name(item))
                num += 1

        elif type(node) in (ecflow.Suite, ecflow.Family, ):
            for item in node.nodes:
                if type(item) in (ecflow.Family, ):
                    self._children[num] = ContainerNode(item)
                elif type(item) in (ecflow.Task, ):
                    self._children[num] = LeafNode(item)
                else:
                    self._children[num] = LeafNode(item)
                keys.append(get_name(item))
                num += 1

            if 0:
                num += decorate(node, self, keys)

        elif type(node) in (ecflow.Task, ):
            pass

        elif 1:
            raise Exception(type(node))
        else:
            self._children[0] = EmptyNode(
                self, parent=self, key=None, depth=num)
        self.dir_count = num + 1
        # print(keys)
        return keys

    def load_child_node(self, key):
        # index = self.get_child_index(key)
        node = self.get_value()
        num = 0
        if type(node) in (ecflow.Client, ):
            node = node.get_defs()

        if type(node) in (ecflow.Task, ):
            return EmptyNode(None)

        if type(node) in (ecflow.Suite, ecflow.Family):
            num = 0
            for item in node.nodes:
                if num == key:
                    node = item
                    break
                if item.name() == key:
                    node = item
                    break
                num += 1

        if key is None:
            return EmptyNode(None)

        if type(node) in (ecflow.Defs, ):
            path = dir_sep()
            for item in node.suites:
                if num == key:
                    node = item
                    break
                if get_name(item) == key:
                    node = item
                    break
                num += 1
            return ContainerNode(node, parent=self)

        # path = os.path.join(get_name(node), key)
        if type(node) in (ecflow.Suite, ecflow.Family, ):
            return ContainerNode(node, parent=self)

        return LeafNode(node, parent=self)

    def load_widget(self):
        return ContainerNodeW(self)


class EcflowBrowser(object):
    palette = [
        ('body', 'black', 'light gray'),
        ('flagged', 'black', 'dark green', ('bold', 'underline')),
        ('focus', 'light gray', 'dark blue', 'standout'),
        ('flagged focus', 'yellow', 'dark cyan',
         ('bold', 'standout', 'underline')),
        ('head', 'yellow', 'black', 'standout'),
        ('foot', 'light gray', 'black'),
        ('key', 'light cyan', 'black', 'underline'),
        ('title', 'white', 'black', 'bold'),
        ('dirmark', 'black', 'dark cyan', 'bold'),
        ('flag', 'dark gray', 'light gray'),
        ('error', 'dark red', 'light gray'),

        ('queued', 'dark gray', 'light blue'),
        ('submitted', 'dark gray', 'light cyan'),
        ('active', 'dark gray', 'light green'),
        ('complete', 'dark gray', 'yellow'),
        ('aborted', 'dark gray', 'dark red'),
        ('unknown', 'dark gray', 'light gray'),
        ('suspended', 'dark gray', 'brown'),
        ('RUNNING', 'dark gray', 'light green'),
        ('HALTED', 'dark gray', 'dark cyan'),  # purple
        ('SHUTDOWN', 'dark gray', 'light red'),  # pink'),
    ]

    footer_text = [
        ('title', "Directory Browser"), "    ",
        ('key', "UP"), ",", ('key', "DOWN"), ",",
        ('key', "PAGE UP"), ",", ('key', "PAGE DOWN"),
        "  ",
        ('key', "SPACE"), "  ",
        ('key', "+"), ",",
        ('key', "-"), "  ",
        ('key', "LEFT"), "  ",
        ('key', "HOME"), "  ",
        ('key', "END"), "  ",
        ('key', "Q"),
    ]

    def __init__(self, server, suites):
        host, port = server.split('@')
        self.ci = ecflow.Client(host, int(port))
        self.ci.ch_register(False, suites)
        self.ci.sync_local()
        # print(self.ci.get_defs())
        self.start_time = time.time()

        # cwd = os.getcwd()
        cwd = self.ci.get_defs()
        store_initial_cwd(dir_sep())
        self.header = urwid.Text("")
        self.listbox = urwid.TreeListBox(urwid.TreeWalker(ContainerNode(cwd)))
        self.listbox.offset_rows = 1
        self.footer = urwid.AttrWrap(urwid.Text(self.footer_text),
                                     'foot')
        self.view = urwid.Frame(
            urwid.AttrWrap(self.listbox, 'body'),
            header=urwid.AttrWrap(self.header, 'head'),
            footer=self.footer)

    def main(self):
        """Run the program."""

        self.loop = urwid.MainLoop(self.view, self.palette,
                                   unhandled_input=self.unhandled_input)
        self.loop.run()

        # on exit, write the flagged filenames to the console
        names = [escape_filename_sh(x) for x in get_flagged_names()]
        print(" ".join(names))

    def unhandled_input(self, k):
        # update display of focus directory
        if k in ('q', 'Q'):
            raise urwid.ExitMainLoop()

        if k in ('z', 'Z', ):
            self.ci.sync_local()

        global STATUS_KIND
        if k in ('3', "3", 3):
            STATUS_KIND = 3

        elif k in ('1', "1", 1):
            STATUS_KIND = 1

        elif k in ('f', "F"):  # full
            STATUS_KIND = 0


def main(server, suites):
    EcflowBrowser(server, suites).main()


_widget_cache = {}


def add_widget(path, widget):
    _widget_cache[path] = widget


def get_flagged_names():
    lst = []
    for w in list(_widget_cache.values()):
        if w.flagged:
            node = w.get_node().get_value()
            lst.append(get_name(node))
    return lst


_initial_cwd = []


def store_initial_cwd(name):
    global _initial_cwd
    _initial_cwd = name.split(dir_sep())


def escape_filename_sh(name):
    """Return a hopefully safe shell-escaped version of a filename."""

    # check whether we have unprintable characters
    for ch in name:
        if ord(ch) < 32:
            # found one so use the ansi-c escaping
            return escape_filename_sh_ansic(name)

    # all printable characters, so return a double-quoted version
    name.replace('\\', '\\\\')
    name.replace('"', '\\"')
    name.replace('`', '\\`')
    name.replace('$', '\\$')
    return '"' + name + '"'


def escape_filename_sh_ansic(name):
    """Return an ansi-c shell-escaped version of a filename."""

    out = []
    # gather the escaped characters into a list
    for ch in name:
        if ord(ch) < 32:
            out.append("\\x%02x" % ord(ch))
        elif ch == '\\':
            out.append('\\\\')
        else:
            out.append(ch)

    # slap them back together in an ansi-c quote  $'...'
    return "$'" + "".join(out) + "'"


SPLIT_RE = re.compile(r'[a-zA-Z]+|\d+')


def alphabetize(s):
    L = []
    import itertools
    for isdigit, group in itertools.groupby(
            SPLIT_RE.findall(s), key=lambda x: x.isdigit()):
        if isdigit:
            for n in group:
                L.append(('', int(n)))
        else:
            L.append((''.join(group).lower(), 0))
    return L


def dir_sep():
    return getattr(os.path, 'se[', '/')


kinds = {ecflow.Defs: "D",
         ecflow.Client: "C",
         ecflow.Suite: "S",
         ecflow.Family: "F",
         ecflow.Task: "T",
         ContainerNode: "X",
         LeafNode: "L",
         str: "S",
         }


if __name__ == '__main__':
    if len(sys.argv) == 2:
        node = sys.argv[1]
    else:
        raise Exception("ecflow_urwid.py localhost@2500/test")
    server, suites = node.split('/')
    main(server, [suites])

"""
python ecflow_urwid.py localhost@2500/test

"""



#!/usr/bin/env python2.7
#////////1/////////2/////////3/////////4/////////5/////////6/////////7/////////8
# Name        :
# Author      : Avi
# Revision    : $Revision: #10 $
#
# Copyright 2009- ECMWF.
# This software is licensed under the terms of the Apache Licence version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
# In applying this licence, ECMWF does not waive the privileges and immunities
# granted to it by virtue of its status as an intergovernmental organisation
# nor does it submit to any jurisdiction.
#////////1/////////2/////////3/////////4/////////5/////////6/////////7/////////8

# =============================================================================
# Code for testing *any* definition
#   Since any ad hoc definition will reference local directories in the
#   ECF_ variables, we need to remove them and inject our own.
#
#   This script is re-runnable, and hence will delete suites in the server
#   matching those in the input definition. Hence it is best to use this 
#   script with a *test* server to avoid accidentally deleting existing suites 
#   of the same name.
# =============================================================================
import ecflow
import os       # for getenv
import shutil   # used to remove directory tree
import argparse # for argument parsing     

def delete_variables_affecting_job_generation(node): 
    """delete customer related ECF variables, these will point to directories
       that don't exist. Its ok we will regenerate our own local ones"""
    var = node.find_variable("ECF_HOME")
    if not var.empty() :  
        node.delete_variable("ECF_HOME")    
    var = node.find_variable("ECF_FILES")
    if not var.empty() :  
        node.delete_variable("ECF_FILES")    
    var = node.find_variable("ECF_INCLUDE")
    if not var.empty() :  
        node.delete_variable("ECF_INCLUDE")    
    var = node.find_variable("ECF_JOB_CMD")
    if not var.empty() :  
        node.delete_variable("ECF_JOB_CMD")    
    var = node.find_variable("ECF_KILL_CMD")
    if not var.empty() :  
        node.delete_variable("ECF_KILL_CMD")    
    var = node.find_variable("ECF_STATUS_CMD")
    if not var.empty() :  
        node.delete_variable("ECF_STATUS_CMD")    
    var = node.find_variable("ECF_OUT")
    if not var.empty() :  
        node.delete_variable("ECF_OUT")    
      
def traverse_container(node_container):
    """Recursively traverse definition node hierarchy and delete
       the variables that affect job generation.
    """
    delete_variables_affecting_job_generation(node_container)
    for node in node_container.nodes:
        delete_variables_affecting_job_generation(node)
        if not isinstance(node, ecflow.Task):
            traverse_container(node)  
    
if __name__ == "__main__":
    
    default_port = "3141"
    if "ECF_PORT" in os.environ:
        default_port = os.environ["ECF_PORT"]

    default_host = "localhost"
    if "ECF_HOST" in os.environ:
        default_host  = os.environ["ECF_HOST"]

    DESC = """Will allow any definition to be loaded and played on the server
            This is done by:
            o Remove existing ECF_ variables that affect job generation. 
              i.e. variables that refer to customer specific directories are removed
            o Allows ECF_HOME to specified, defaults to cwd + /CUSTOMER/ECF_HOME
            o Generates the scripts(.ecf files) automatically based on the definition.
              i.e. if a task has events,meters,labels then the client request for these are
              automatically injected in the generated .ecf script files
            o Will clear out existing data both on disk and on the server to allow 
              multiple re-runs of this script. ** If this is an issue please use
              a test server **
            o All suites are put into a suspended state. This allows the GUI to resume them
            o The server is restarted and suites are begun
            This programs assumes that ecflow module is accessible.
            """    
    PARSER = argparse.ArgumentParser(description=DESC,  
                                     formatter_class=argparse.RawDescriptionHelpFormatter)
    PARSER.add_argument('defs_file', 
                        help="The definition file")
    PARSER.add_argument('--host', default=default_host,   
                        help="The name of the host machine, defaults to ECF_HOST otherwise 'localhost'")
    PARSER.add_argument('--port', default=default_port,   
                        help="The port on the host, defaults to ECF_PORT otherwise uses 3141")
    PARSER.add_argument('--path', default="/",   
                        help="replace only the node path in the suite")
    PARSER.add_argument('--ecf_home', default=os.getcwd() + "/CUSTOMER/ECF_HOME",
                        help="Directory to be used for generated scripts(ECF_HOME), defaults to ./CUSTOMER/ECF_HOME")
    PARSER.add_argument('--verbose', nargs='?', default=False, const=True, type=bool,
                        help="Show verbose output")
    ARGS = PARSER.parse_args()
    ARGS.defs_file = os.path.expandvars(ARGS.defs_file) # expand references to any environment variables
    print(ARGS) 
    
    # If running on local work space, use /libs/pyext/test/data/CUSTOMER/ECF_HOME as ecf_home
    if ARGS.verbose:
        print("Using ECF_HOME=" + ARGS.ecf_home)
         
    if ARGS.verbose: 
        print("\nloading the definition from the input arguments(" + ARGS.defs_file + ")\n")
    try:
        DEFS = ecflow.Defs(ARGS.defs_file)
    except RuntimeError as ex:
        print("   ecflow.Defs(" + ARGS.defs_file + ") failed:\n" + str(ex))
        sys.exit(1)
    
    if ARGS.verbose: 
        print("remove test data associated with the DEFS, so we start fresh, Allows rerun")
    for suite in DEFS.suites:
        dir_to_remove = ARGS.ecf_home + suite.get_abs_node_path()
        if ARGS.verbose: 
            print("   Deleting directory: " + dir_to_remove + "\n")
        shutil.rmtree(dir_to_remove, True)  
        
    if ARGS.verbose: 
        print("remove remote reference to ECF_HOME and ECF_INCLUDE, since we inject or own\n")
    for suite in DEFS.suites:
        traverse_container(suite)
  
    if ARGS.verbose: 
        print("add variables required for script generation, for all suites\n")
    DEFS.add_variable("ECF_HOME", ARGS.ecf_home)
    DEFS.add_variable("SLEEP", "10")  # not strictly required since default is 1 second
    DEFS.add_variable("ECF_INCLUDE", ARGS.ecf_home + "/includes")


    if ARGS.verbose: 
        print("Place all suites into suspended state, so they can be started by the GUI\n")  
    for suite in DEFS.suites:
        suite.add_defstatus(ecflow.DState.suspended)
    
    if ARGS.verbose: 
        #ecflow.PrintStyle.set_style(ecflow.Style.STATE)
        print(DEFS)

    if ARGS.verbose: 
        print("Generating script files(.ecf) from the definition")
    DEFS.generate_scripts()

    if ARGS.verbose: 
        print("\nchecking script file generation, pre-processing & variable substitution\n")
    msg = DEFS.check_job_creation()       
    assert len(msg) == 0, msg

    # ===========================================================================
    CL = ecflow.Client(ARGS.host, ARGS.port)
    try:
        if ARGS.verbose: 
            print("check server " + ARGS.host + ":" + ARGS.port + " is running")
        CL.ping() 

        if ARGS.verbose: 
            print("Server is already running. re-start the server")
        CL.restart_server() 

        if ARGS.verbose: 
            print("Remove suites associated with this DEFS, allows rerun *******************************************")
        for suite in DEFS.suites:
            try:
                CL.delete(suite.get_abs_node_path(), True)
            except RuntimeError as ex:
                pass # For first run this will fail, hence ignore
        
        if ARGS.verbose: 
            print("Load the definition into " + ARGS.host + ":" + ARGS.port)
        if ARGS.path == "/":
            CL.load(DEFS) 
        else:
            CL.replace(ARGS.path, DEFS)

        if ARGS.verbose: 
            print("Begin all suites. They should be suspended.")
        print("Loaded suites:")
        for suite in DEFS.suites:
            CL.begin_suite(suite.name())
            print("   " + suite.name())
        print("into server " + ARGS.host \
                             + ":" + ARGS.port + ", please view the playable suites in the GUI")
    except RuntimeError as ex:
        print("Error: " + str(ex))



#////////1/////////2/////////3/////////4/////////5/////////6/////////7/////////8
# Name        :
# Author      : Avi
# Revision    : $Revision: #10 $
#
# Copyright 2009- ECMWF.
# This software is licensed under the terms of the Apache Licence version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
# In applying this licence, ECMWF does not waive the privileges and immunities
# granted to it by virtue of its status as an intergovernmental organisation
# nor does it submit to any jurisdiction.
#////////1/////////2/////////3/////////4/////////5/////////6/////////7/////////8

import time
import argparse # for argument parsing  
from ecflow import Client, debug_build

def timing_decorator(function_to_time):
    def scafold(*function_args):
        # function_args is a tuple, the second argument is the suite name
        start = time.time()
        function_return = function_to_time(*function_args)
        end = time.time()
        print('%s function took %0.3f ms for %s' % (function_to_time.__name__, (end-start)*1000.0, function_args[1]))
        return function_return
    return scafold

@timing_decorator
def sync_local(ci,suite_name):
    ci.sync_local()

if __name__ == "__main__":
    
    DESC = """This test is use to show the performance of sync local on the given server
              It should show that it is quicker to register and then sync_local()
              as this limits the amount of data that needs to be down loaded from the
              server.
              Usage:
                   TestSyncLocalPerf.py --host <hostname> --port <portname>  
            """    
    PARSER = argparse.ArgumentParser(description=DESC,  
                                     formatter_class=argparse.RawDescriptionHelpFormatter)
    PARSER.add_argument('--host', default="localhost",   
                        help="The name of the host machine, defaults to 'localhost'")
    PARSER.add_argument('--port', default="3141",   
                        help="The port on the host, defaults to 3141")
    ARGS = PARSER.parse_args()
    print(ARGS)   
     
    print("####################################################################")
    print(("Test performance of sync local using " + Client().version() + " debug build(" + str(debug_build()) +")"))
    print("####################################################################")

    CL = Client(ARGS.host, ARGS.port)
    sync_local(CL,"* All suites *")   # timing for downloading all suites
    print("")

    suites = CL.get_defs().suites
    for suite in suites:
        CL.ch_register(False,[suite.name()])
        
        sync_local(CL,suite.name()) # time this
        
        CL.ch_drop()  # drop the last registered handle
            


#
# Copyright 2009- ECMWF.
#
# This software is licensed under the terms of the Apache Licence version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
# In applying this licence, ECMWF does not waive the privileges and immunities
# granted to it by virtue of its status as an intergovernmental organisation
# nor does it submit to any jurisdiction.
#

import os
import ecflow as ecf
import itertools as it

import ecflow_test_util as Test


def can_create_mirror_from_parameters():
    mirror = ecf.MirrorAttr("name", "r_path", "r_host", "r_port", "polling", True, "auth")
    assert mirror.name() == "name"
    assert mirror.remote_path() == "r_path"
    assert mirror.remote_host() == "r_host"
    assert mirror.remote_port() == "r_port"
    assert mirror.polling() == "polling"
    assert mirror.ssl() == True
    assert mirror.auth() == "auth"


def can_create_mirror_from_default_parameters_0():
    mirror = ecf.MirrorAttr("name", "r_path")
    assert mirror.name() == "name"
    assert mirror.remote_path() == "r_path"
    assert mirror.remote_host() == "%ECF_MIRROR_REMOTE_HOST%"
    assert mirror.remote_port() == "%ECF_MIRROR_REMOTE_PORT%"
    assert mirror.polling() == "%ECF_MIRROR_REMOTE_POLLING%"
    assert mirror.ssl() == False
    assert mirror.auth() == "%ECF_MIRROR_REMOTE_AUTH%"


def can_create_mirror_from_default_parameters_1():
    mirror = ecf.MirrorAttr("name", "r_path", "r_host")
    assert mirror.name() == "name"
    assert mirror.remote_path() == "r_path"
    assert mirror.remote_host() == "r_host"
    assert mirror.remote_port() == "%ECF_MIRROR_REMOTE_PORT%"
    assert mirror.polling() == "%ECF_MIRROR_REMOTE_POLLING%"
    assert mirror.ssl() == False
    assert mirror.auth() == "%ECF_MIRROR_REMOTE_AUTH%"


def can_create_mirror_from_default_parameters_2():
    mirror = ecf.MirrorAttr("name", "r_path", "r_host", "r_port")
    assert mirror.name() == "name"
    assert mirror.remote_path() == "r_path"
    assert mirror.remote_host() == "r_host"
    assert mirror.remote_port() == "r_port"
    assert mirror.polling() == "%ECF_MIRROR_REMOTE_POLLING%"
    assert mirror.ssl() == False
    assert mirror.auth() == "%ECF_MIRROR_REMOTE_AUTH%"


def can_create_mirror_from_default_parameters_3():
    mirror = ecf.MirrorAttr("name", "r_path", "r_host", "r_port", "polling")
    assert mirror.name() == "name"
    assert mirror.remote_path() == "r_path"
    assert mirror.remote_host() == "r_host"
    assert mirror.remote_port() == "r_port"
    assert mirror.polling() == "polling"
    assert mirror.ssl() == False
    assert mirror.auth() == "%ECF_MIRROR_REMOTE_AUTH%"


def can_create_mirror_from_default_parameters_4():
    mirror = ecf.MirrorAttr("name", "r_path", "r_host", "r_port", "polling", True)
    assert mirror.name() == "name"
    assert mirror.remote_path() == "r_path"
    assert mirror.remote_host() == "r_host"
    assert mirror.remote_port() == "r_port"
    assert mirror.polling() == "polling"
    assert mirror.ssl() == True
    assert mirror.auth() == "%ECF_MIRROR_AUTH%"


def can_add_mirror_to_task():
    suite = ecf.Suite("s1")

    family = ecf.Family("f1")
    suite.add_family(family)

    task = ecf.Task("f1")
    family.add_task(task)

    mirror = ecf.MirrorAttr("name", "r_path", "r_host", "r_port", "polling", True, "auth")
    task.add_mirror(mirror)
    assert len(list(task.mirrors)) == 1

    actual = list(task.mirrors)[0]
    assert actual.name() == "name"
    assert actual.remote_path() == "r_path"
    assert actual.remote_host() == "r_host"
    assert actual.remote_port() == "r_port"
    assert actual.ssl() == True
    assert actual.auth() == "auth"


def can_embed_mirror_into_task():
    suite = ecf.Suite("s1")

    family = ecf.Family("f1")
    suite.add_family(family)

    task = ecf.Task("f1", ecf.MirrorAttr("name", "r_path", "r_host", "r_port", "polling", True, "auth"))
    assert len(list(task.mirrors)) == 1

    actual = list(task.mirrors)[0]
    assert actual.name() == "name"
    assert actual.remote_path() == "r_path"
    assert actual.remote_host() == "r_host"
    assert actual.remote_port() == "r_port"
    assert actual.ssl() == True
    assert actual.auth() == "auth"


def cannot_have_multiple_mirrors_in_single_task():
    suite = ecf.Suite("s1")

    family = ecf.Family("f1")
    suite.add_family(family)

    try:
        task = ecf.Task("f1", ecf.MirrorAttr("name", "r_path", "r_host", "r_port", "polling", True, "auth"),
                        ecf.MirrorAttr("another", "r_path", "r_host", "r_port", "polling", True, "auth"))
        assert False, "Expected exception indicating multiple mirrors are not allowed in a task"
    except RuntimeError as e:
        assert True

def can_check_job_creation_with_mirror():
    defs = ecf.Defs()

    suite = ecf.Suite("s")
    defs.add_suite(suite)

    family = ecf.Family("f")
    suite.add_family(family)

    task = ecf.Task("t");
    family.add_task(task)

    mirror = ecf.MirrorAttr("name", "r_path", "r_host", "r_port", "polling", True, "auth")
    task.add_mirror(mirror)

    defs.check_job_creation()


if __name__ == "__main__":
    Test.print_test_start(os.path.basename(__file__))

    can_create_mirror_from_parameters()
    can_create_mirror_from_default_parameters_0()
    can_create_mirror_from_default_parameters_1()
    can_create_mirror_from_default_parameters_2()
    can_create_mirror_from_default_parameters_3()
    can_add_mirror_to_task()
    can_embed_mirror_into_task()
    cannot_have_multiple_mirrors_in_single_task()
    can_check_job_creation_with_mirror()

    print("All tests pass")



from .ecflow import *

# Name        :
# Author      : Avi
# Revision    : $Revision: #10 $ 
#
# Copyright 2009- ECMWF.
# This software is licensed under the terms of the Apache Licence version 2.0 
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0. 
# In applying this licence, ECMWF does not waive the privileges and immunities 
# granted to it by virtue of its status as an intergovernmental organisation 
# nor does it submit to any jurisdiction.  

"""
The ecFlow python module
"""

__version__ = '5.13.8'

# http://stackoverflow.com/questions/13040646/how-do-i-create-documentation-with-pydoc



# Copyright 2009-2019 ECMWF.
# This software is licensed under the terms of the Apache Licence version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
# In applying this licence, ECMWF does not waive the privileges and immunities
# granted to it by virtue of its status as an intergovernmental organisation
# nor does it submit to any jurisdiction.

import argparse  # for argument parsing
import sys

import ecflow


if __name__ == "__main__":

    DESC = """Will list variables for any node
              Usage:
                Example1: List all the server variables
                   ListVariables.py --host cca --port 4141 --path /
 
                Example2: List the variables for the given node
                   ListVariables.py --host cca --port 4141 --path /path/to/node
            """
    PARSER = argparse.ArgumentParser(
        description=DESC, formatter_class=argparse.RawDescriptionHelpFormatter
    )
    PARSER.add_argument(
        "--host",
        default="localhost",
        help="The name of the host machine, defaults to 'localhost'",
    )
    PARSER.add_argument(
        "--port", default="3141", help="The port on the host, defaults to 3141"
    )
    PARSER.add_argument(
        "--path", default="/", help="The path to the node. i.e. /suite/family/task"
    )
    ARGS = PARSER.parse_args()
    # print(ARGS)

    # ===========================================================================
    CL = ecflow.Client(ARGS.host, ARGS.port)
    try:
        CL.ping()

        # get the incremental changes, and merge with defs stored on the Client
        CL.sync_local()

        # check to see if definition exists in the server
        defs = CL.get_defs()
        if defs is None:
            print("No suites found, exiting...")
            sys.exit(0)

        # print(defs)
        if ARGS.path == "/":
            for var in defs.server_variables:
                print("edit " + var.name() + " '" + var.value() + "'")
            for var in defs.user_variables:
                print("edit " + var.name() + " '" + var.value() + "'")
        else:
            node = defs.find_abs_node(ARGS.path)
            if node == None:
                print("No node found at path " + ARGS.path)
            else:
                for var in node.variables:
                    print("edit " + var.name() + " '" + var.value() + "'")

    except RuntimeError as ex:
        print("Error: " + str(ex))
        print("Check host and port number are correct.")



#!/usr/bin/env python
# -*- coding= UTF-8 -*-
#////////1/////////2/////////3/////////4/////////5/////////6/////////7/////////8
# Name        :
# Author      : Avi
# Revision    : $Revision: #10 $
#
# Copyright 2009- ECMWF.
# This software is licensed under the terms of the Apache Licence version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
# In applying this licence, ECMWF does not waive the privileges and immunities
# granted to it by virtue of its status as an intergovernmental organisation
# nor does it submit to any jurisdiction.
#////////1/////////2/////////3/////////4/////////5/////////6/////////7/////////8
""" tkinter example with ecflow
"""

import tkinter as tki
from tkinter import *
import queue
import sys
import time
import os
import pwd
from threading import Thread
try: from scrolledlist import ScrolledList
except: 
    loc = "http://infohost.nmt.edu/tcc/help/lang/python/examples/scrolledlist"
    raise Exception("wget %s/scrolledlist.py; # Thanks NMT")

loc = "/usr/local/apps/ecflow/current/lib/python2.7/site-packages/ecflow"
sys.path.append(loc)
loc = "/usr/local/lib/python2.7/site-packages/ecflow"
sys.path.append(loc)
import ecflow as ec

PROGRAM_NAME = "ecflowview-overview"
BUTTON_FONT = ('times', 12)
MONO_FONT = ('lucidatypewriter', 14, 'bold')
DEBUG = 0
COLORS = {"aborted": "red",
          "active":  "green",
          "submitted": "cyan",
          "complete": "yellow",
          "suspended": "orange",
          "queued": "blue",
          "unknwon": "grey"}
running = [True]

ONCE = False


def show_check(one):
    print(one)
    sep = ','
    if sep not in one: return
    exp = one.split(sep)
    if len(exp) != 4: return
    host, qid = exp[2], exp[3]
    check = os.getenv("CHECK_TASK", "check_task.py")
    global ONCE
    if check is not None:
        print(check +  " -n %s -j %s" % (host, qid))
    elif ONCE:
        print("#WAR: export CHECK_TASK=$HOME/bin/check_task.py; overview.py $ECF_HOST@$ECF_PORT,")
        ONCE = True


class Label(object):
    """ a class to encapsulate what was a global variable"""
    inst = None

    def __init__(self, item): Label.inst = item

    @classmethod
    def update(cls):
        if Label.inst is None:
            return
        Label.inst.set(time.strftime("%a, %d %b %Y %H:%M:%S"))


class MenuBar(tki.Frame):
    """ guess"""

    def __init__(self, parent):
        tki.Frame.__init__(self, parent)
        self.parent = parent
        self.__helpButton = self.__createHelp()
        self.__helpButton.grid(row=0, column=3)
        self.__quitButton = tki.Button(self, text='Quit', font=BUTTON_FONT,
                                       command=root.destroy)
        self.__quitButton.grid(row=0, column=1)
        self.__updateButton = tki.Button(self, text='Update', font=BUTTON_FONT,
                                         command=parent.update)
        self.__updateButton.grid(row=0, column=2)

    def quit(self, event=None):
        self.parent.quit()

    def __createHelp(self):
        mb = tki.Menubutton(self, font=BUTTON_FONT,
                            relief=tki.RAISED, text='Help')
        menu = tki.Menu(mb)
        mb['menu'] = menu
        url = "https://confluence.ecmwf.int/display/ECFLOW/Documentation"

        def url1(): self.__url(url=url)

        def url2(): self.__url(url="http://effbot.org/tkinterbook/")
        menu.add_command(command=url1, label="confluence tutorial?")
        menu.add_command(command=url2, label="tkinter?")
        return mb

    def __url(self, url=None):
        if url is not None:
            os.system("${BROWSER:=firefox} %s" % url)


class TaskList(tki.Frame):
    """ a class to host all tasks"""
    # WIDTH = w / 30
    # LINES = h / 10

    def __init__(self, parent, kind):
        tki.Frame.__init__(self, parent, 
                           # side="bottom", 
                           # fill="both", 
                           # expand=True,
                           # sticky="nsew",
        )
        self.WIDTH = 80
        self.LINES = 80
        self.__kind = kind
        self.__callback = None
        #self.__label = tki.Label(
        #    self, font=BUTTON_FONT, background=COLORS[kind], text=kind)
        #self.__label.grid(row=0, column=0, sticky=tki.W)
        select = tki.Button(self, text=kind,
                                 font=BUTTON_FONT,
                                 background=COLORS[kind],
                                 command=self.select)
        select.grid(row=1, column=0, sticky=tki.W)

        selectall = tki.Button(self, text="select all",
                                    font=BUTTON_FONT,
                                    # background=COLORS[kind],
                                    command=self.select_all)
        selectall.grid(row=2, column=0, sticky=tki.W)

        self.__scrolledList = ScrolledList(
            self, 
            # sticky="nsew",
            width=self.WIDTH, 
            height=self.LINES,  # selectmode=tki.EXTENDED,
            callback=self.__callback)
        self.__scrolledList.grid(row=3, column=0)

    def select_all(self):
        res = set()
        item = self.__scrolledList.listbox.get(0, tki.END)
        for one in item:
            print(one)

    def select(self):
        res = set()
        selected = self.__scrolledList.listbox.curselection()
        for i in selected:
            item = self.__scrolledList.listbox.get(i)
            res.add(item)
            for one in res: 
                show_check(one)
                
    def insert(self, path): self.__scrolledList.append(path)

    def clear(self): self.__scrolledList.clear()


class PaceKeeper():
    """ get regular updates"""
    PACE = 60

    def __init__(self, item, queue):
        self._item = item
        Thread(target=self.process, args=(queue, running)).start()

    def process(self, queue, running):
        while running:
            queue.put(self._item.update)
            time.sleep(self.PACE)

    def run(self): self.update()

    def update(self, verbose=False):
        while True:
            self._item.update()
            time.sleep(self.PACE)


class Client(object):
    """ a class to focus on client-ecFlow-server comm"""

    def __init__(self, one="localhost@31415"):
        # try: nick, hhh = one.split("-")
        # except: hhh = one; nick = None
        if "@" in one:
            host, port = one.split("@")
        # except: 
        else: host = "localhost"; port = 31415
        self.nick = one
        # if nick is None: self.nick = "%s@%s" % (host, port)
        print("# client creation", self.nick, host, port)
        self.client = ec.Client(host, port)

    def process(self, win):
        # Label.update()
        self.client.sync_local()
        defs = self.client.get_defs()
        if len(defs) == 0:
            print("# %s-%: empty content" % (self.host, self.port))
        # Label.update()
        for suite in defs.suites:
            self.process_nc(suite, win)

    def process_nc(self, node, win):
        for item in node.nodes:
            if isinstance(item, ec.Task):
                self.process_node(item, win)
            else:
                self.process_nc(item, win)

    def process_node(self, item, wins):
        for kind, win in list(wins.items()):
            status = "%s" % item.get_state()
            if status != kind:
                continue
            change = item.get_state_change_time()
            line = "#%s:%s,%s" % (
                self.nick, item.get_abs_node_path(), change)
            try:
                import check_task as ct
                qid = str(ct.display_var(item, "qid"))
                host = ct.display_var(item, "ECF_JOB_CMD")
                line += ",%s,%s" % (host, qid)
            except: pass
            win.insert(line)

try:
    sys.path.append('/usr/local/apps/sms/lib/')
    import cdp
    NOCDP = 0
except:
    NOCDP = 1


class ClientSMS(Client):
    """ example of class derivation to connect another server-scheduler"""
    init = 0

    def __init__(self, host="localhost", port=31415, nick=None):
        if NOCDP:
            return
        if not ClientSMS.init:
            cdp.cdp_init([""], 0)
            ClientSMS.init = 1
        else:
            pass
        node = cdp.sms_node()
        tree = cdp.sms_info_pointed(node)
        cdp.sms_numbers(port, 0)
        timeout = 60
        passwd = "1"
        user = get_username()

        if DEBUG:
            print("#MSG: login:", nick, host, port, user)
        handle, rc = cdp.sms_client_login(host, user, passwd,
                                          timeout, port, tree)
        self.handle = handle
        self.host = host
        self.port = port
        self.num = -1
        self.nick = nick
        if nick is None:
            self.nick = "%s@%d" % (self.host, self.port)

    def process_node(self, node, wins):
        if node.type != cdp.NODE_TASK:
            kid = node.kids
            while kid:
                self.process_node(kid, wins)
                kid = kid.__next__
        else:
            for kind, win in list(wins.items()):
                state = "%s" % ClientSMS.status(node)
                if state != kind:
                    continue
                path = cdp.sms_node_full_name(node)
                win.insert("%s:%s" % (self.nick, path))
                if DEBUG:
                    print(self.host, self.port, path, state)

    def process(self, win):
        rc = cdp.sms_client_news(self.handle, self.num)
        if rc or self.num == -1:
            Label.update()
            state = cdp.sms_client_status2(self.handle, self.num)
            cdp.sms_client_status(self.handle, state, self.num)

            item = cdp.sms_status2node(state)
            if not item:
                return
            self.process_node(item, win)
            Label.update()
            cdp.sms_node_free(item)

    @classmethod
    def status(cls, node):
        s_status = {
            -1: "unknown",
            0: "unknown",
            1: "suspended",
            2: "complete",
            3: "queued",
            4: "submitted",
            5: "active",
            6: "aborted",
            7: "shutdown",
            8: "halted",
            9: "unknown", }
        return s_status[node.status]

    def __exit__(self):
        cdp.sms_client_logout(self.handle)
        print("# logout from", self.nick)

    def __delete__(self):
        cdp.sms_client_logout(self.handle)
        print("# logout from", self.nick)



class Application(tki.Frame):
    """ main """

    def __init__(self, parent=None, client=None, queue=None):
        if parent is None: parent = tki.Tk()
        w, h = parent.winfo_screenwidth(), parent.winfo_screenheight()
        tki.Frame.__init__(self, parent)
        if client is None:
            raise Exception("#ERR: please provide a client...\n./overview.py $ECF_HOST@$ECF_PORT")
        elif type(client) in (set, tuple):
            self.__clients = client
        else:
            self.__clients = [client]
        self.__queue = queue
        # width = 640;         height = 780
        width = w;         height = h
        self.canvas = tki.Canvas(width=width, height=height, bg='black')
        self.grid()
        self.createWidgets()
        self.canvas.after(50, self.check_queue)

        self.bind_all("<space>", self.update)
        # self.bind_all("Control_q", self.quit)
        # self.bind_all("Control_z", self.resize)
        self.bind_all("<Control-q>", self.quit)
        self.bind_all("<Control-z>", self.resize)

    def resize(self, event=None): print("New size is: {}x{}".format(event.width, event.height))

    def quit(self, event=None):
        print("quit...")
        self.destroy()
        # global running; running = [ False, ]
        # sys.exit(0)

    def createWidgets(self):
        global root
        root = self
        rowx = 1
        var = tki.StringVar(self)
        glob = Label(root)
        # glob = Label(var)
        self.__menuBar = MenuBar(root)
        self.__menuBar.grid(row=0, column=0, sticky=tki.W)
        self.label = tki.Label(root, textvariable=var) # Label.inst)
        self.label.grid(row=0, column=2, sticky=tki.E)
        self.__wins = dict()
        rowx += 1
        colx = 0
        kinds = ("active", "aborted", "submitted")
        for kind in kinds:
            self.__wins[kind] = TaskList(root, kind)
            self.__wins[kind].grid(row=rowx, column=colx,
                                   sticky=tki.S + tki.E + tki.W)
            colx += 1
        self.update()

    def check_queue(self):
        try:
            self.__queue.get(block=False)
        except queue.Empty:
            pass
        else:
            self.update()
        self.canvas.after(50, self.check_queue)

    def update(self, event=None):
        self.label.update()
        for kind, win in list(self.__wins.items()):
            win.clear()
        for client in self.__clients:
            if type(client) == list:
                for clt in client:
                    clt.process(self.__wins)
            else:
                client.process(self.__wins)


def get_username(): return pwd.getpwuid(os.getuid())[0]


def get_uid(): return pwd.getpwnam(get_username()).pw_uid


if __name__ == '__main__':
    try:
        port = 1500 + int(get_uid())
    except:
        port = 31415

    if len(sys.argv) > 1:
        clients = []
        for num in range(1, len(sys.argv)):            
            sep = ','
            arg = sys.argv[num]
            if sep in arg:
                for one in arg.split(sep):
                    client = Client(one)
            else:
                client = Client(arg)
            clients.append(client)
    else: 
        one = "localhost%d" % port
        print("# using", one)
        clients = [Client(one), ]

    queue = queue.Queue()
    app = Application(client=clients, queue=queue)
    app.master.title(PROGRAM_NAME)
    app.columnconfigure(0, weight=1)
    app.rowconfigure(0, weight=1)
    PaceKeeper(app, queue)
    app.mainloop()

"""
python ./overview.py localhost@31415,
"""



#////////1/////////2/////////3/////////4/////////5/////////6/////////7/////////8
# Name        :
# Author      : Avi
# Revision    : $Revision: #10 $
#
# Copyright 2009- ECMWF.
# This software is licensed under the terms of the Apache Licence version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
# In applying this licence, ECMWF does not waive the privileges and immunities
# granted to it by virtue of its status as an intergovernmental organisation
# nor does it submit to any jurisdiction.
#////////1/////////2/////////3/////////4/////////5/////////6/////////7/////////8
# SCRATCH test for ecflow python api

from ecflow import Alias, AttrType, Autocancel, CheckPt, ChildCmdType, Client, Clock, Cron, DState, Date, Day, Days, \
                  Defs, Ecf, Event, Expression, Family, FamilyVec, File, Flag, FlagType, FlagTypeVec, InLimit, \
                  JobCreationCtrl, Label, Late, Limit, Meter, Node, NodeContainer, NodeVec, PartExpression, PrintStyle, \
                  Repeat, RepeatDate, RepeatDay, RepeatEnumerated, RepeatInteger, RepeatString, SState, State, Style, \
                  Submittable, Suite, SuiteVec, Task, TaskVec, Time, TimeSeries, TimeSlot, Today, UrlCmd, Variable, \
                  VariableList, Verify, WhyCmd, ZombieAttr, ZombieType, ZombieUserActionType, Trigger, Complete, Edit, Defstatus
import unittest 
import sys
import os

class TestCron(unittest.TestCase):
    def setUp(self):
        cron = Cron()
        cron.set_week_days( [0,1,2,3,4,5,6] )
        cron.set_days_of_month( [1,2,3,4,5,6] )
        cron.set_months( [1,2,3,4,5,6] )
        cron.set_time_series( "+00:00 23:00 00:30" )
        self.cron1 =  cron
        
        cron2 = Cron()
        cron2.set_time_series(1, 30) # default relative = false, added in release 4.0.7
        self.cron2 = cron2

        cron3 = Cron()
        cron3.set_week_days([0, 1, 2, 3, 4, 5, 6 ])
        cron3.set_time_series(1, 30, True)
        self.cron3 = cron3
    
        cron4 = Cron()
        cron4.set_week_days([0, 1, 2, 3, 4, 5, 6])
        cron4.set_time_series("00:30 01:30 00:01")
        self.cron4 = cron4

        cron5 = Cron()
        cron5.set_week_days([0, 1, 2, 3, 4, 5, 6])
        cron5.set_time_series("+00:30")
        self.cron5 = cron5
        
        start = TimeSlot(0, 0)
        finish = TimeSlot(23, 0)
        incr = TimeSlot(0, 30)
        time_series = TimeSeries(start, finish, incr, True)
        cron6 = Cron( time_series )
        cron6.set_week_days([0, 1, 2, 3])
        self.cron6 = cron6
        
        cron7 = Cron()
        cron7.set_week_days( [0,1,2] )                       # This cant overlap with last_week_days_of_the_month
        cron7.set_last_week_days_of_the_month( [3,4,5,6] )   # This cant overlap with days_of_week
        cron7.set_days_of_month( [1,2,3,4,5,6] )
        cron7.set_months( [1,2,3,4,5,6] )
        cron7.set_time_series( "+00:00 23:00 00:30" )
        cron7.set_last_day_of_the_month()
        self.cron7 = cron7

    def test_me0(self):
        cron = Cron("+00:00 23:00 00:30", days_of_week=[0,1,2,3,4,5,6],days_of_month=[1,2,3,4,5,6], months=[1,2,3,4,5,6])
        self.assertEqual(self.cron1, cron, "cron not equal\n" + str(self.cron1) + "\n" + str(cron))
                
        cron = Cron("01:30")
        self.assertEqual(self.cron2, cron, "cron not equal\n" + str(self.cron2) + "\n" + str(cron))

        cron = Cron("+01:30",days_of_week=[0,1,2,3,4,5,6])
        self.assertEqual(self.cron3, cron, "cron not equal\n" + str(self.cron3) + "\n" + str(cron))

        cron = Cron("00:30 01:30 00:01",days_of_week=[0,1,2,3,4,5,6])
        self.assertEqual(self.cron4, cron, "cron not equal\n" + str(self.cron4) + "\n" + str(cron))

        cron = Cron("+00:30",days_of_week=[0,1,2,3,4,5,6])
        self.assertEqual(self.cron5, cron, "cron not equal\n" + str(self.cron5) + "\n" + str(cron))

        cron = Cron("+00:00 23:00 00:30",days_of_week=[0,1,2,3])
        self.assertEqual(self.cron6, cron, "cron not equal\n" + str(self.cron6) + "\n" + str(cron))

        cron = Cron("+00:00 23:00 00:30",days_of_week=[0,1,2],                   # This cant overlap with last_week_days_of_the_month
                                         last_week_days_of_the_month=[3,4,5,6],  # This cant overlap with days_of_week
                                         days_of_month=[1,2,3,4,5,6],
                                         months=[1,2,3,4,5,6],
                                         last_day_of_the_month=True)
        self.assertEqual(self.cron7, cron, "cron not equal\n" + str(self.cron7) + "\n" + str(cron))
 
    def test_fail(self):
        with self.assertRaises(RuntimeError):
            cron = Cron("");  # empty
        with self.assertRaises(RuntimeError):
            cron = Cron("", days_of_week=[0,1,2,3,4,5,6],days_of_month=[1,2,3,4,5,6], months=[1,2,3,4,5,6]); # empty
        with self.assertRaises(RuntimeError):
            cron = Cron("0023", days_of_week=[0,1,2,3,4,5,6],days_of_month=[1,2,3,4,5,6], months=[1,2,3,4,5,6]) # bad time
        with self.assertRaises(RuntimeError):
            cron = Cron("00:10", days_of_weekss=[0,1,2,3,4,5,6],days_of_month=[1,2,3,4,5,6], months=[1,2,3,4,5,6]) # bad kw,days_of_weekss
        with self.assertRaises(RuntimeError):
            cron = Cron("00:10", days_of_week=[0,1,2,3,4,5,6],days_of_months=[1,2,3,4,5,6], months=[1,2,3,4,5,6]) # bad kw, days_of_months
        with self.assertRaises(RuntimeError):
            cron = Cron("00:10", days_of_week=[0,1,2,3,4,5,6],days_of_month=[1,2,3,4,5,6], monthss=[1,2,3,4,5,6]) # bad kw, monthss
        with self.assertRaises(RuntimeError):
            cron = Cron("00:10", days_of_week="0,1,2,3,4,5,6",days_of_month=[1,2,3,4,5,6], month=[1,2,3,4,5,6])  # bad kw, expect vector of string
        
if __name__ == "__main__":
    unittest.main()
    print("All Tests pass")



#////////1/////////2/////////3/////////4/////////5/////////6/////////7/////////8
# Name        :
# Author      : Avi
# Revision    : $Revision: #10 $
#
# Copyright 2009- ECMWF.
# This software is licensed under the terms of the Apache Licence version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
# In applying this licence, ECMWF does not waive the privileges and immunities
# granted to it by virtue of its status as an intergovernmental organisation
# nor does it submit to any jurisdiction.
#////////1/////////2/////////3/////////4/////////5/////////6/////////7/////////8

#  code for testing pointers and hierarchy in python

import os
from ecflow import Suite, Family, Task, Defs, Client, debug_build
import ecflow_test_util as Test

def check_then_auto_add_extern(defs):
    error_msg = defs.check() 
    print(error_msg)
    assert len(error_msg) !=0,"Expect error in checks\n" + str(defs)
    
    defs.auto_add_externs(True)
    error_msg = defs.check()
    assert len(error_msg) == 0,"Expect check to pass after auto add extern\n" + error_msg + "\n" + str(defs)
    
if __name__ == "__main__":   
    Test.print_test_start(os.path.basename(__file__))

    defs = Defs()
    error_msg = defs.check();  assert len(error_msg) == 0,"Expect empty defs to pass check"
    suite = defs.add_suite("ext");

    f1 = suite.add_family("f1")
    f1.add_task("t").add_trigger("/a/b/c/d == complete");     check_then_auto_add_extern(defs)
    f1.add_task("t1").add_trigger("/a/b/c/d/e:event == set"); check_then_auto_add_extern(defs)
    f1.add_task("t2").add_trigger("/a/b/c/d/x:event");        check_then_auto_add_extern(defs)     
    f1.add_task("t3").add_trigger("/a/b/c/d/y:meter le 30");  check_then_auto_add_extern(defs)  
    f1.add_task("t4").add_trigger("/a/b/c/d/z<flag>late");    check_then_auto_add_extern(defs)   
    
    f2 = suite.add_family("f2")
    f2.add_inlimit("hpcd","limits");                           check_then_auto_add_extern(defs)
    f2.add_task("t").add_inlimit("sg1","/suiteName");          check_then_auto_add_extern(defs)
    f2.add_task("t1").add_inlimit("hpcd","/obs/limits");       check_then_auto_add_extern(defs) 
    f2.add_task("t2").add_inlimit("c1a","/limits");            check_then_auto_add_extern(defs)     
    print (defs)
    print("All Tests pass")



#!/usr/bin/env python
#////////1/////////2/////////3/////////4/////////5/////////6/////////7/////////8
# Name        :
# Author      : Avi
# Revision    : $Revision: #10 $
#
# Copyright 2009- ECMWF.
# This software is licensed under the terms of the Apache Licence version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
# In applying this licence, ECMWF does not waive the privileges and immunities
# granted to it by virtue of its status as an intergovernmental organisation
# nor does it submit to any jurisdiction.
#////////1/////////2/////////3/////////4/////////5/////////6/////////7/////////8
import sys
import getopt
try: import ecflow
except:
        sys.path.append("/usr/local/apps/ecflow/current/lib/python2.7/site-packages/ecflow")
        import ecflow 
import ecflow  as ec
import os
import sys
""" a simple program to convert an expanded definition file to 
py script using ecf.py"""

class Indent:
    """This class manages indentation, for use with context manager
    It is used to correctly indent the definition node tree hierarchy
    """
    _pos = 0
    _step = 3
    def __init__(self):
        Indent._pos += Indent._step
    def __del__(self):
        Indent._pos -= Indent._step
    @classmethod
    def indent(cls, loc=''):
        out = ''
        for i in range(Indent._pos):
            if loc is None: 
                out += ' ' 
            elif type(loc) == str: 
                loc += ' '
            else: 
                loc.write(' ')
        if type(loc) == str: 
            return loc
        return out

def adds(line=0):
    if line is None: 
        return ""

    return Indent.indent() + line + "\n"

def add(line, echo=0):
    if line is None: 
        return ""
    elif echo: 
            return Indent.indent() + line 
    else: 
            return Indent.indent() + line + "\n"

class DefFormat(object):
    def __init__(self, defs): 
        self.defs = defs

    def process_attr(self, node, end=False):
        # print node.get_abs_node_path()
        res = ""
        ind = Indent()
        defstatus = node.get_defstatus()
        if defstatus: 
            if defstatus != ec.DState.queued:
                res += add("Defstatus('%s')," % defstatus)

        item = node.get_autocancel()
        if item: 
            line = "%s" % item
            line = line.replace("autocancel ", "")
            res += add("Autocancel('%s')," % line)

        item = node.get_repeat()
        if not item.empty(): 
            line = "%s" % item
            full = line.split()
            kind = full[1]
            name = full[2]
            try:
                beg  = full[3]
                try: end  = full[4]
                except: end = beg
                if "#" in end: end = beg
            except: beg = 1; end = 1
            by = 1
            if len(full) == 6: by = full[5]

            if kind in ("integer", "date"):
                res += add(
                    "Repeat(kind='%s', name='%s', start=%s, end=%s, step=%s)," 
                    % ( kind, name, beg, end, by))
            elif kind in ("day", ):
                res += add(
                    "Repeat(kind='%s', name='%s', step=%s)," 
                    % ( kind, name, by))
            elif kind in ("string", "enumerated"):
                line = "%s" % item
                line = line.replace("repeat %s %s" % (kind, name), "")
                line.replace('"', '')
                res += add("Repeat(kind='%s', name='%s', start=%s)," % (
                        kind, name, line.split()))
             # FIXME string enum

        item = node.get_late()
        if item: 
            line = "%s" % item
            line = line.replace("late ", "")
            res +=  add("Late('%s')," % line)

        item = node.get_complete()
        if item: res += add("Complete('%s')," % item)

        item = node.get_trigger()
        if item: 
            res += add("Trigger('%s')," % item)

        for item in node.meters: 
            line = "%s" % item
            dummy, name, beg, end, thr = line.split(" ")
            res += add("Meter('%s', %s, %s, %s)," % (name, beg, end, thr))

        for item in node.events: 
            line = "%s" % item
            line = line.replace("event ", "").strip()
            res += add("Event('%s')," % line)

        for item in node.labels: 
            res += add("Label('%s', '%s')," % (item.name(), item.value()))

        for item in node.limits: 
            val = item.value()
            if val == "0" or val == 0: 
                val = "1"
            res += add("Limit('%s', %d)," % (
                    item.name(), int(val)))

        for item in node.inlimits:
            line = "%s" % item
            line = line.replace("inlimit ", "")
            res += add("InLimit('%s')," % line)

        for item in node.times:  
            line = "%s" % item
            line = line.replace("time ", "")
            res += add("Time('%s')," % line)  

        for item in node.todays: 
            line = "%s" % item
            line = line.replace("today ", "")
            res += add("Today('%s')," % line) 

        for item in node.dates:  
            line = "%s" % item
            line = line.replace("date ", "")            
            res += add("Date('%s')," % line)  

        for item in node.days:   
            line = "%s" % item
            line = line.replace("day ", "")            
            
            res += add("Day('%s')," % item)   
        for item in node.crons:  
            line = "%s" % item
            line = line.replace("cron ", "")            
            res += add("Cron('%s')," % line)  

        var = "Variables("
        vind = Indent()            
        num = 0
        for item in node.variables: 
            sep="'"
            if sep in item.value(): sep="\""
            var += ind.indent("\n") + item.name() + \
                "= %s%s%s," % (sep, item.value(), sep)
            num += 1
        del vind        
        if num: res += add(var + "),")

        del ind
        if len(res) == 0: 
            return None
        return res

    def process(self, node=None, inc=0):
        STREAM = 1
        out = ""
        if node is None:
            num = 0
            out += "# ecf.ECF_MODE = 'sms'"
            out += "suites = []"
            for suite in self.defs.suites:
                if STREAM: out += "suite%d = Suite('%s').add(" % (num, suite.name())
                else: out += "s = Suite('%s')"% (suite.name()) + "; suites.append(s);"
                out += self.process(suite)
                out += ")"
                num += 1
            else: out += "#WAR no suite to process"

            if num > 1: 
                if 0: raise BaseException("no more than one suite at a time")
                out += "### WARNING: more than one suite defined"

        elif isinstance(node, ec.Suite): 
            if not STREAM: out += "s.add(\n"
            add( self.process_attr(node),1 )
            if STREAM: ind = Indent()
            for kid in node.nodes: out += self.process(kid)
            if STREAM: del ind

        elif isinstance(node, ec.Family): 
            many = ""; one = ""; post = "" # circumvent limitation to 255 items
            if inc % 200 == 0: 
                one += "("; post = "),"
                if inc > 200: many = "),"
            if STREAM: 
                add( many + one + "Family('%s').add(" % node.name(), 1)
                res = self.process_attr(node)
            else:
                out += ")\ncur = Family('%s')" % node.name()
                out += "fam.add(cur); fam = cur; fam.add("
                res = self.process_attr(node)

            if STREAM: 
                if res is not None: 
                        out += Indent.indent(res) # add(res, 1)
                ind = Indent()
            num = 0
            for kid in node.nodes: self.process(kid, num)            ; num += 1
            if STREAM: 
                del ind
                add( Indent.indent(post + "), # endfamily %s" % node.name()), 1 )
            out += "# %s " % num

        elif isinstance(node, ec.Task): 
            res = self.process_attr(node)
            if res is None:  
                add( "Task('%s')," % node.name(), 1)
                # print Indent.indent("Task('%s')," % node.name())
            else: add( "Task('%s').add(\n" % node.name() +
                       "%s%s)," % (res, Indent.indent()), 1)

        return out

    def main(self):
        return self.head() + self.process() + self.tail()

    def head(self):
        return """#!/usr/bin/env python
# NOTICE: this file was originally created with def2def.py
import sys
# sys.path.append('../o/def')
# sys.path.append('.')
sys.path.append('/home/ma/emos/def/o/def')
from ecf import *
import ecf  as ecf
# import inc_emos as ie
"""

    def tail(self):
        return """

if __name__ == '__main__':
    defs = Defs()
    defs.add(suite0); 
    defs.auto_add_externs(True)
    if 1: 
      import cli_proc, ecf
      cli_proc.process(ie.Seed(defs), compare=False)
    else:
       port = 1500 + int(get_uid()) # when started with ecflow_start.sh # ECMWF
       node = "localhost"
       path = '/' + suite0.name()
       if 0: # test job creation
         job_ctrl = ecflow.JobCreationCtrl()                    
         defs.check_job_creation(job_ctrl)

       print "replacing %s on %s@%s" % (path, node, port)
       client = ecf.Client("localhost", port)
       client.replace(path, defs)
"""

def usage():
        print("""def2def.py
  -f file    [filename]
  -s string  [description of the suite]
  -c client  [host@post:/path/to/node]
  -h help

""")

def test(quiet=1):
        import os
        num = 0
        if quiet: quiet = " > /dev/null "
        else: quiet = ""
        for cmd in (
                        "./def2def.py    || : ", # usage
                        "./def2def.py -h || : ", # usage
                        "./def2def.py -s 'suite s; family f; task t'",
                        "./def2def.py -s 'suite s\n family f\n task t'",
                        "./def2def.py -p vsms2@32222/o ",
                        "./def2def.py -f ../o.exp",
                        "./def2def.py    ../o.exp"):
                # print "#", cmd
                # os.system(cmd + " > /dev/null && echo '#' OK%d" % num)
                os.system(cmd + quiet
                          + " && echo '#' OK%d " % num + cmd
                          + " || echo '#' NOK%d " %  num + cmd)
                num += 1

def process(desc=None, path=None, fname=None):
        if desc is not None: 
                import tempfile
                print("#desc: ", desc) # , "\n"
                # import unicodedata
                #unic = unicodedata.normalize('NFKD', desc).encode('ascii', 'ignore')
                #print "#unic: ", unic # , "\n"
                USER = os.getenv("USER")
                try:
                        with tempfile.NamedTemporaryFile(mode="w+t", dir="/tmp/%s" % USER) as temp:
                                if type(desc) == list:
                                        for line in desc:
                                                temp.write(line)
                                                temp.write("\n")
                                elif type(desc) == str:
                                        temp.write(desc.rstrip())
                                temp.flush()
                                # defs = ec.Defs(temp.name)
                                # if DEBUG: print defs
                                return process(fname= temp.name)
                except RuntimeError as e:    
                        out = "#WAR: failed: " + str(e)
                        # print "temp, NOK"; sys.exit(1)
                        print(out)
                        return out
                # return "%s" % defs
        elif path is not None: 
                import re
                node, rem = path.split("@")
                port, suite = rem.split('/')[0:2]
                print(node, port, suite)
                client = ec.Client(node, port)
                client.ch_register(False, [ suite ])
                client.sync_local()
                defs = client.get_defs()
        elif fname is not None: pass
        elif fname is None and len(sys.argv) > 1:
                fname = sys.argv[1]
        else: usage(); sys.exit(2)

        if fname:
                defs = ec.Defs(fname)
        defs.auto_add_externs(True)
        if DEBUG: print(defs)
        return DefFormat(defs).main()

DEBUG = 0
if __name__ == '__main__':
        opts, args = getopt.getopt(sys.argv[1:], "gqhf:s:p:t",
                                   ["debug", "help", "file", "string", "path", "test"])
        fname = None
        desc  = None
        path  = None
        QUIET = 0
        DEBUG = 0
        for o, a in opts:
                if o in ("-g", "debug"): DEBUG = 1
                if o in ("-h", "help"): usage(); sys.exit(2)
                if o in ("-f", "file"): fname = a
                if o in ("-s", "string"): desc = a
                if o in ("-p", "path"): path = a
                if o in ("-t", "test"): test(QUIET); sys.exit(0)
                if o in ("-q", "quiet"): QUIET = 1

        print(process(desc, path, fname))
# ./def2def.py -q -t 
# ./def2def.py -t 



#////////1/////////2/////////3/////////4/////////5/////////6/////////7/////////8
# Name        :
# Author      : Avi
# Revision    : $Revision: #10 $
#
# Copyright 2009- ECMWF.
# This software is licensed under the terms of the Apache Licence version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
# In applying this licence, ECMWF does not waive the privileges and immunities
# granted to it by virtue of its status as an intergovernmental organisation
# nor does it submit to any jurisdiction.
#////////1/////////2/////////3/////////4/////////5/////////6/////////7/////////8
#  code for testing client code in python
import time
import os
import pwd
import sys  # determine python version
from datetime import datetime
import shutil   # used to remove directory tree

from ecflow import Defs, Clock, DState, Event, Style, State, PrintStyle, File, Client, SState, File, debug_build
import ecflow_test_util as Test


def ecf_includes(): return File.source_dir() + "/libs/pyext/test/data/python_includes"

def create_defs(name,the_port):
    defs = Defs()
    suite_name = name
    if len(suite_name) == 0: suite_name = "s1"
    suite = defs.add_suite(suite_name);
    
    ecfhome = Test.ecf_home(the_port);
    suite.add_variable("ECF_HOME", ecfhome);
    suite.add_variable("ECF_INCLUDE", ecf_includes());
    
    prefix_job_cmd = ""
    if 'PYTHONPATH' in os.environ:
        prefix_job_cmd = "export PYTHONPATH=" + os.environ['PYTHONPATH'] +";" 
    if 'LD_LIBRARY_PATH' in os.environ:
        prefix_job_cmd += "export LD_LIBRARY_PATH=" + os.environ['LD_LIBRARY_PATH'] +";"
   
    # sys.version_info is a tuple containing (major,minor,micro,releaselevel,serial)
    # releaselevel = alpha beta candidate final
    if (sys.version_info > (3, 0)):
        suite.add_variable("ECF_JOB_CMD",prefix_job_cmd + "python3 %ECF_JOB% 1> %ECF_JOBOUT% 2>&1")
    else:
        suite.add_variable("ECF_JOB_CMD",prefix_job_cmd + "python %ECF_JOB% 1> %ECF_JOBOUT% 2>&1")

    family = suite.add_family("f1")
    t1 = family.add_task("t1")
    t1.add_event("event_fred")
    t1.add(Event("event_set",True)) # ECFLOW-1526
    t1.add_meter("meter", 0, 100)
    t1.add_label("label_name", "value")
    t1.add_queue("q1",["1","2","3"])

    family.add_task("t2")  # test wait
    family.add_task("t3").add_trigger("t1:q1 >= 3 and t1:event_fred and t1:event_set == clear") # wait on queue q1 and events
    family.add_task("t4").add_trigger("t1:name1 == 1 and t1:name2 == 2 and t1:name3 == 3 and t1:name4 == 4") # test ECFLOW-1573
 
    defs.auto_add_externs(True) # because variable name1,name2,name3,name4  are not added until t1 is active.(i.e. runtime)
    return defs;
    
def wait_for_suite_to_complete(ci,suite_name):
    count = 0
    while 1:
        count += 1
        ci.sync_local() # get the changes, synced with local defs
        suite = ci.get_defs().find_suite(suite_name)
        assert suite is not None, " Expected to find suite " + suite_name + ":\n" + str(ci.get_defs())
        if suite.get_state() == State.complete:
            break;
        if suite.get_state() == State.aborted:
            print(ci.get_defs());
            assert False," Suite aborted \n"  
        time.sleep(2)
        if count > 20:
            assert False, suite_name + " aborted after " + str(count) + " loops, printing defs:\n" + str(ci.get_defs())
        
    ci.log_msg("Looped " + str(count) + " times")
    
def test_python_child_api(ci):   
    suite_name = "test_python_child_api"         
    print("\n" + suite_name + " " + ci.get_host() + ":" + str(ci.get_port()))
    print(" ECF_HOME(" + Test.ecf_home(ci.get_port()) + ")")
    print(" ECF_INCLUDES(" + ecf_includes() + ")")
    ci.delete_all(True)     
    defs = create_defs(suite_name,ci.get_port())  
    suite = defs.find_suite(suite_name)
    suite.add_defstatus(DState.suspended)

    # create the ecf file /test_python_child_api/f1/t1
    ecf_home = Test.ecf_home(ci.get_port())
    test_home = ecf_home + "/" + suite_name
    dir = test_home + "/f1"
    
    # on cray creating recursive directories can fail, try again. yuk
    try:
        if not os.path.exists(dir): os.makedirs(dir)
    except:
        try:
            if not os.path.exists(dir): os.makedirs(dir)
        except:
            # try breaking down
            if not os.path.exists(ecf_home): os.makedirs(ecf_home)
            new_dir = ecf_home + "/test_python_child_api"
            if not os.path.exists(new_dir): os.makedirs(new_dir)
            new_dir = ecf_home + "/test_python_child_api/f1"
            if not os.path.exists(new_dir): os.makedirs(new_dir)
    if not os.path.exists(dir): os.makedirs(dir)

    # record the log file with the test
    ci.new_log(test_home + "/" + suite_name + ".log")

    server_version = ci.server_version()
    file = dir + "/t1.ecf"
    contents = "%include <head.py>\n\n"
    contents += "with Client(True) as ci:\n" # Here True means add variables durint init and remove them on complete ECFLOW-1573
    contents += "    print('   doing some work: t1.ecf')\n"
    contents += "    if ci.version() != '" + server_version + "':\n"
    contents += "        assert False, 'Client and server versions different'\n"
    contents += "    ci.child_event('event_fred')      # set the event\n"
    contents += "    ci.child_event('event_set',False) # clear the event ECFLOW-1526\n"
    contents += "    ci.child_meter('meter',100)\n"
    contents += "    ci.child_label('label_name','100')\n"
    contents += "    step = ci.child_queue('q1','active')\n"
    contents += "    assert step == '1','expected first step to be 1'\n"
    contents += "    step = ci.child_queue('q1','complete',step)\n"
    contents += "    step = ci.child_queue('q1','active')\n"
    contents += "    assert step == '2','expected second step to be 2'\n"
    contents += "    step = ci.child_queue('q1','complete',step)\n"
    contents += "    step = ci.child_queue('q1','active')\n"
    contents += "    assert step == '3','expected third step to be 3'\n"
    contents += "    step = ci.child_queue('q1','complete',step)\n"
    contents += "    step = ci.child_queue('q1','active')\n"
    contents += "    assert step == '<NULL>','expected <NULL? for end of queue'\n"
    contents += "    print('   Finished event,meter,label and queue child commands')\n"
    open(file,'w').write(contents)
    print(" Created file " + file)
      
    # create the ecf file /test_python_child_api/f1/t2
    file = dir + "/t2.ecf"
    contents = "%include <head.py>\n\n"
    contents += "with Client() as ci:\n"
    contents += "    print('   Waiting for /test_python_child_api/f1/t1 == complete')\n"
    contents += "    ci.child_wait('/test_python_child_api/f1/t1 == complete')\n"
    contents += "    print('   Finished waiting')\n"
    open(file,'w').write(contents)
    print(" Created file " + file)
    
    # create the ecf file /test_python_child_api/f1/t3
    file = dir + "/t3.ecf"
    contents = "%include <head.py>\n\n"
    contents += "with Client() as ci:\n"
    contents += "    print('   Running t3.ecf')\n"
    open(file,'w').write(contents)
    print(" Created file " + file)
    
    # create the ecf file /test_python_child_api/f1/t4
    file = dir + "/t4.ecf"
    contents = "%include <head.py>\n\n"
    contents += "with Client() as ci:\n"
    contents += "    print('   Running t4.ecf')\n"
    open(file,'w').write(contents)
    print(" Created file " + file)
            
    ci.restart_server()
    ci.load(defs)          
    ci.begin_all_suites()
    print(" Running the test, wait for suite to complete ...")  
    ci.run("/test_python_child_api", False)

    wait_for_suite_to_complete(ci,suite_name);
    
    defs.save_as_defs(os.path.join(ecf_home,suite_name, suite_name + ".def"))

    if not Test.debugging():
        print(" Test OK: removing directory ",test_home)
        shutil.rmtree(test_home,ignore_errors=True)      

#///////////////////////////////////////////////////////////////////////////////////////        

if __name__ == "__main__":
    Test.print_test_start(os.path.basename(__file__))

    with Test.Server() as ci:
    
        server_version = ci.server_version();
        print("Running ecflow server version " + server_version)
        print("Running ecflow client version " + ci.version() )
        assert ci.version() == server_version, "Client version not same as server version"

        PrintStyle.set_style( Style.STATE ) # show node state 
        test_python_child_api(ci)  
        print("\nAll Tests pass ======================================================================")    



import os
from ecflow import (
    Defs,
    Suite,
    Family,
    Task,
    Edit,
    Trigger,
    Complete,
    Event,
    Meter,
    Time,
    Day,
    Date,
    Label,
    RepeatString,
    RepeatInteger,
    RepeatDate,
)


def create_family_f4():
    return Family(
        "f4",
        Edit(SLEEP=2),
        RepeatString("NAME", ["a", "b", "c", "d", "e", "f"]),
        Family(
            "f5",
            RepeatInteger("VALUE", 1, 10),
            Task(
                "t1",
                RepeatDate("DATE", 20101230, 20110105),
                Label("info", ""),
                Label("date", ""),
            ),
        ),
    )


print("Creating suite definition")
home = os.path.join(os.getenv("HOME"), "course")
defs = Defs(Suite("test", Edit(ECF_INCLUDE=home, ECF_HOME=home), create_family_f4()))
print(defs)
print("Checking job creation: .ecf -> .job0")
print(defs.check_job_creation())

print("Checking trigger expressions")
assert len(defs.check()) == 0, defs.check()

print("Saving definition to file 'test.def'")
defs.save_as_defs("test.def")



#!/usr/bin/env python2.7
#////////1/////////2/////////3/////////4/////////5/////////6/////////7/////////8
# Name        :
# Author      : Avi
# Revision    : $Revision: #10 $
#
# Copyright 2009- ECMWF.
# This software is licensed under the terms of the Apache Licence version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
# In applying this licence, ECMWF does not waive the privileges and immunities
# granted to it by virtue of its status as an intergovernmental organisation
# nor does it submit to any jurisdiction.
#////////1/////////2/////////3/////////4/////////5/////////6/////////7/////////8
import ecflow
import argparse # for argument parsing  
import sys   

class Indentor:
    """This class manages indentation, 
    It is used to correctly indent the definition node tree hierarchy
    """
    _index = 0
    def __init__(self):
        Indentor._index += 1
    def __del__(self):
        Indentor._index -= 1
    @classmethod
    def indent(cls):
        for i in range(Indentor._index):
            print(' ', end=' ')
     
class DefsTraverser:
    """Traverse the ecflow.Defs definition and writes to standard out.
    
    Ecflow has the following hierarchy::
       Task   --> Submittable   -->Node
       Family --> NodeContainer -->Node
       Suite  --> NodeContainer -->Node
    
    This demonstrates that all nodes in the node tree and all attributes are accessible.
    Additionally the state data is also accessible. This class will write state data as
    comments. If the definition was returned from the server, it allows access to latest 
    snapshot of the state data held in the server. 
    """
    def __init__(self,defs):
        assert (isinstance(defs,ecflow.Defs)),"Expected ecflow.Defs as first argument"
        self.__defs = defs
        
    def do_print(self):
        for extern in self.__defs.externs:
            self.__println("extern " + extern)
        for suite in self.__defs.suites:
            self.__print("suite ")
            self.__print_node(suite)
            clock = suite.get_clock()
            if clock:
                indent = Indentor()
                self.__println(str(clock))
                del indent
            self.__print_nc(suite)
            self.__println("endsuite")  
 
    def __print_nc(self,node_container):
        indent = Indentor()
        for node in node_container.nodes:
            if isinstance(node, ecflow.Task):
                self.__print("task ")
                self.__print_node(node)
                self.__print_alias(node)
            else: 
                self.__print("family ")
                self.__print_node(node)
                self.__print_nc(node)
                self.__println("endfamily")
        del indent

    def __print_alias(self,task):
       indent = Indentor()
       for alias in task.nodes:
           self.__print("alias ")
           self.__print_node(alias)
           self.__println("endalias")
       del indent

    def __print_node(self,node):
        print(node.name() + " # state:" + str(node.get_state())) 
        
        indent = Indentor()
        defStatus = node.get_defstatus()
        if defStatus != ecflow.DState.queued: 
            self.__println("defstatus " + str(defStatus))
            
        autocancel = node.get_autocancel()
        if autocancel: self.__println(str(autocancel))
        
        repeat = node.get_repeat()
        if not repeat.empty(): self.__println(str(repeat)  + " # value: " + str(repeat.value()))
    
        late = node.get_late()
        if late: self.__println(str(late) + " # is_late: " + str(late.is_late()))

        complete_expr = node.get_complete()
        if complete_expr:
            for part_expr in complete_expr.parts:
                trig = "complete "
                if part_expr.and_expr(): trig = trig + "-a "
                if part_expr.or_expr():  trig = trig + "-o "
                self.__print(trig)
                print(part_expr.get_expression())
        trigger_expr = node.get_trigger()
        if trigger_expr:
            for part_expr in trigger_expr.parts:
                trig = "trigger "
                if part_expr.and_expr(): trig = trig + "-a "
                if part_expr.or_expr():  trig = trig + "-o "
                self.__print(trig)
                print(part_expr.get_expression()) 
                
        for var in node.variables:    self.__println("edit " + var.name() + " '" + var.value() + "'")
        for meter in node.meters:     self.__println(str(meter) + " # value: " + str(meter.value()))
        for event in node.events:     self.__println(str(event) + " # value: " + str(event.value()))
        for label in node.labels:     self.__println(str(label) + " # value: " + label.new_value())
        for limit in node.limits:     self.__println(str(limit) + " # value: " + str(limit.value()))
        for inlimit in node.inlimits: self.__println(str(inlimit))
        for the_time in node.times:   self.__println(str(the_time))
        for today in node.todays:     self.__println(str(today))   
        for date in node.dates:       self.__println(str(date))  
        for day in node.days:         self.__println(str(day))   
        for cron in node.crons:       self.__println(str(cron))    
        for verify in node.verifies:  self.__println(str(verify))
        for zombie in node.zombies:   self.__println(str(zombie))
        for queue in node.queues:     self.__println(str(queue))
        
        del indent

    def __print(self,the_string):
        Indentor.indent()
        print(the_string, end=' ')

    def __println(self,the_string):
        Indentor.indent()
        print(the_string)
 
if __name__ == "__main__":
    
    DESC = """Will list all the nodes and attributes in the definition
              Usage:
                Example1: List all the states
                   printdefs.py --host cca --port 4141  
            """    
    PARSER = argparse.ArgumentParser(description=DESC,  
                                     formatter_class=argparse.RawDescriptionHelpFormatter)
    PARSER.add_argument('--host', default="localhost",   
                        help="The name of the host machine, defaults to 'localhost'")
    PARSER.add_argument('--port', default="3141",   
                        help="The port on the host, defaults to 3141")
    ARGS = PARSER.parse_args()
    #print ARGS    
    
    # ===========================================================================
    CL = ecflow.Client(ARGS.host, ARGS.port)
    try:
        CL.ping() 

        # get the incremental changes, and merge with defs stored on the Client 
        CL.sync_local()
        
        # check to see if definition exists in the server
        defs = CL.get_defs()
        if len(defs) == 0 :
            print("No suites found, exiting...")
            sys.exit(0) 
            
        # print defs;
        defs_traverser = DefsTraverser(defs)
        defs_traverser.do_print()

    except RuntimeError as ex:
        print("Error: " + str(ex))
        print("Check host and port number are correct.")



#
# Copyright 2009- ECMWF.
#
# This software is licensed under the terms of the Apache Licence version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
# In applying this licence, ECMWF does not waive the privileges and immunities
# granted to it by virtue of its status as an intergovernmental organisation
# nor does it submit to any jurisdiction.
#

import ecflow_test_util as ect
import ecflow as ecf
import unittest
import sys
import os


class TestNode(unittest.TestCase):

    def setUp(self):
        self.defs = ecf.Defs()
        self.suite = self.defs.add_suite("suite");

        self.suite.add_variable("VARIABLE", "value");

        self.family = self.suite.add_family("family")
        self.family.add_repeat(ecf.RepeatDate("REPEAT", 20010101, 20010102, 1))

        self.task = self.family.add_task("task")

    def test_is_able_to_retrieve_suite_generated_variables_using_variable_list(self):
        vars = ecf.VariableList()
        self.suite.get_generated_variables(vars)

        names = [v.name() for v in vars]
        self.assertIn("SUITE", names)
        self.assertIn("ECF_DATE", names)
        self.assertIn("ECF_CLOCK", names)
        self.assertIn("ECF_TIME", names)
        self.assertIn("ECF_JULIAN", names)

        self.assertIn("YYYY", names)
        self.assertIn("DD", names)
        self.assertIn("MM", names)
        self.assertIn("DAY", names)
        self.assertIn("MONTH", names)
        self.assertIn("DATE", names)
        self.assertIn("TIME", names)
        self.assertIn("DOW", names)
        self.assertIn("DOY", names)

    def test_is_able_to_retrieve_suite_generated_variables_using_python_list(self):
        vars = self.suite.get_generated_variables()

        names = [v.name() for v in vars]
        self.assertIn("SUITE", names)
        self.assertIn("ECF_DATE", names)
        self.assertIn("ECF_CLOCK", names)
        self.assertIn("ECF_TIME", names)
        self.assertIn("ECF_JULIAN", names)

        self.assertIn("YYYY", names)
        self.assertIn("DD", names)
        self.assertIn("MM", names)
        self.assertIn("DAY", names)
        self.assertIn("MONTH", names)
        self.assertIn("DATE", names)
        self.assertIn("TIME", names)
        self.assertIn("DOW", names)
        self.assertIn("DOY", names)

    def test_is_able_to_retrieve_family_generated_variables_using_variable_list(self):
        vars = ecf.VariableList()
        self.family.get_generated_variables(vars)

        names = [v.name() for v in vars]
        self.assertIn("FAMILY", names)
        self.assertIn("FAMILY1", names)
        self.assertIn("REPEAT", names)
        self.assertIn("REPEAT_YYYY", names)
        self.assertIn("REPEAT_MM", names)
        self.assertIn("REPEAT_DD", names)
        self.assertIn("REPEAT_DOW", names)
        self.assertIn("REPEAT_JULIAN", names)

    def test_is_able_to_retrieve_family_generated_variables_using_python_list(self):
        vars = self.family.get_generated_variables()

        names = [v.name() for v in vars]
        self.assertIn("FAMILY", names)
        self.assertIn("FAMILY1", names)
        self.assertIn("REPEAT", names)
        self.assertIn("REPEAT_YYYY", names)
        self.assertIn("REPEAT_MM", names)
        self.assertIn("REPEAT_DD", names)
        self.assertIn("REPEAT_DOW", names)
        self.assertIn("REPEAT_JULIAN", names)

    def test_is_able_to_retrieve_task_generated_variables_using_variable_list(self):
        vars = ecf.VariableList()
        self.task.get_generated_variables(vars)

        names = [v.name() for v in vars]
        self.assertIn("TASK", names)
        self.assertIn("ECF_JOB", names)
        self.assertIn("ECF_SCRIPT", names)
        self.assertIn("ECF_JOBOUT", names)
        self.assertIn("ECF_TRYNO", names)
        self.assertIn("ECF_RID", names)
        self.assertIn("ECF_PASS", names)
        self.assertIn("ECF_NAME", names)

    def test_is_able_to_retrieve_task_generated_variables_using_python_list(self):
        vars = self.task.get_generated_variables()

        names = [v.name() for v in vars]
        self.assertIn("TASK", names)
        self.assertIn("ECF_JOB", names)
        self.assertIn("ECF_SCRIPT", names)
        self.assertIn("ECF_JOBOUT", names)
        self.assertIn("ECF_TRYNO", names)
        self.assertIn("ECF_RID", names)
        self.assertIn("ECF_PASS", names)
        self.assertIn("ECF_NAME", names)


if __name__ == "__main__":
    unittest.main()
    print("All Tests pass")



#////////1/////////2/////////3/////////4/////////5/////////6/////////7/////////8
# Name        :
# Author      : Avi
# Revision    : $Revision: #10 $
#
# Copyright 2009- ECMWF.
# This software is licensed under the terms of the Apache Licence version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
# In applying this licence, ECMWF does not waive the privileges and immunities
# granted to it by virtue of its status as an intergovernmental organisation
# nor does it submit to any jurisdiction.
#////////1/////////2/////////3/////////4/////////5/////////6/////////7/////////8
from ecflow import Defs, Client, debug_build
import ecflow_test_util as Test
import os

def create_defs_sequentially():
    local_defs = Defs()
    suite = local_defs.add_suite("s1")
    suite.add_variable("Var","value")
    suite.add_meter( "metername3", 0, 100, 50 )
    suite.add_event( 2 )
    suite.add_event( 3 )
    f1 = suite.add_family("f1")
    f1 = f1.add_family("f1")
    t1 = f1.add_task("t1")
    t1.add_variable("fred","jones")
    
    f2 = suite.add_family("f2")
    f1 = f2.add_family("f1")
    t1 = f1.add_task("t1")
    t1.add_variable("fred","jones")

    f3 = suite.add_family("f3")
    t1 = f3.add_task("t1")
    t1.add_variable("fred",    "jones")
    
    suite.add_task("t1").add_variable("fred","jones")
    return local_defs

def create_defs_functionally():
    f_defs = Defs()
    suite = f_defs.add_suite("s1")
    suite.add_variable("Var","value").add_family("f1").add_family("f1").add_task("t1").add_variable("fred","jones")
    suite.add_family("f2").add_family("f1").add_task("t1").add_variable("fred","jones")
    suite.add_meter( "metername3", 0, 100, 50 ).add_family("f3").add_task("t1").add_variable("fred","jones")
    suite.add_event( 2 ).add_event( 3 ).add_task("t1").add_variable("fred","jones")
    return f_defs
    
    
if __name__ == "__main__":
        
    Test.print_test_start(os.path.basename(__file__))

    #
    # Add Nodes functional way
    #
    defs = Defs()
    defs.add_suite("s1").add_task("t1").add_variable("var","v")
    defs.add_suite("s2").add_family("f1").add_task("t1").add_variable("var","v")
    defs.add_suite("s3").add_family("f1").add_family("f2").add_task("t1").add_variable("var","v")
    assert len(defs) == 3,"Expected 3 suites"    
  
    # These test show that although add_variable() returns a node_ptr, its really a suite
    # hence calling add_family/add_task still works.
    assert create_defs_sequentially() == create_defs_functionally(),"Functional suite not as expected "
  
    print("All Tests pass")
    